/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 * Time initialization.
 */
#include <common.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/system.h>
#ifndef IS_FPGA
#include <platform_lib/board/pcb.h>
#endif

DECLARE_GLOBAL_DATA_PTR;

const struct bsp_sysinfo sysinfo = {
#ifdef BOARD_NAME_STRING
	BOARD_NAME_STRING
#else
    "Board: Realtek Demo Board\n"
#endif
};

#ifdef CONFIG_OTP_UTIL
extern int otp_read_bits(unsigned long ofst, unsigned char msb, unsigned char lsb, unsigned long* pval);
#endif

/**
 * @brief checkboard
 *
 * @return 0
 */
int checkboard(void)
{
	printf(sysinfo.board_string);
    return 0;
}

/**
 * @brief board_init
 *
 * @return 0
 */
extern void *env_get_config_boot_param_base(void);
int board_init(void)
{
#ifdef CONFIG_RTD299O
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD299O;
#elif defined(CONFIG_RTD299S)
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD299S;
#elif defined(CONFIG_RTD289X)
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD289X;
#elif defined(CONFIG_RTD287X)
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD287X;
#elif defined(CONFIG_RTD287O)
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD287O;
#elif defined(CONFIG_RTD285X)
	gd->bd->bi_arch_number = MACH_TYPE_RTK_RTD285X;
#endif
	gd->bd->bi_boot_params = (ulong)env_get_config_boot_param_base();//CONFIG_BOOT_PARAM_BASE; /* boot param addr */

	return 0;
}

/**
 * @brief dram_init_banksize
 *
 * @return 0
 */
void dram_init_banksize(void)
{
#ifndef CONFIG_LESS_CONSOLE
	puts("dram_init_banksize()...\n");
#endif

#ifdef CONFIG_RAM_SIZE_FROM_OTP_SETTING
	unsigned int total_size = 0;

	total_size = get_ddr_size();

	if (total_size != 0x0) // valid total size
	{
		gd->bd->bi_dram[0].start = CONFIG_SYS_SDRAM_BASE;
		gd->bd->bi_dram[0].size  = 0x20000000;
		gd->bd->bi_dram[1].start = gd->bd->bi_dram[0].start + gd->bd->bi_dram[0].size;
		gd->bd->bi_dram[1].size  = total_size - gd->bd->bi_dram[0].size;
		return ;
	}
#endif

	gd->bd->bi_dram[0].start = CONFIG_SYS_SDRAM_BASE;
	gd->bd->bi_dram[0].size  = CONFIG_SYS_RAM_DCU1_SIZE;
#if (CONFIG_NR_DRAM_BANKS > 1)
	gd->bd->bi_dram[1].start = CONFIG_SYS_SDRAM_DCU2_BASE;
	gd->bd->bi_dram[1].size  = CONFIG_SYS_RAM_DCU2_SIZE;
#endif
}

int board_eth_init(bd_t *bis)
{
	return 0;
}

/**
 * @brief misc_init_r - Configure Panda board specific configurations
 * such as power configurations, ethernet initialization as phase2 of
 * boot sequence
 *
 * @return 0
 */
int misc_init_r(void)
{
	return 0;
}

/*
 * get_board_rev() - get board revision
 */
u32 get_board_rev(void)
{
	uint revision = 0;

	revision = (uint)simple_strtoul(CONFIG_VERSION, NULL, 16);

	return revision;
}

#ifdef CONFIG_SYS_VIDDQ
int board_viddq_cal(void)
{
#ifdef CONFIG_OTP_UTIL
	unsigned long iddqcheck;
	unsigned long iddqdonebit;

	printf("\033[1;44mdo RTD287O IC setting\033[0m\n");

	otp_read_bits(0x134, 6, 6, &iddqdonebit); //iddq donebit
	otp_read_bits(0x12c, 8, 0, &iddqcheck); //core iddq
	printf("viddq register value is = 0x%lx, 0x%lx\n", iddqdonebit, iddqcheck);

	if (iddqdonebit)
	{
		if(iddqcheck >= 68) //high iddq
		{
			printf("High IDDQ / Low VID, and Core IDDQ= %ld \n", iddqcheck);
			IO_Set(PIN_CORE_VID_0, 0); // (expected 0.9704V)
			IO_Set(PIN_SCPU_VID_0, 0); // (expected 1.0053V)
		//	IO_Set(PIN_SCPU_VID_1, 0);
		}
		else // low iddq
		{
			printf("Low IDDQ / High VID, and Core IDDQ= %ld \n", iddqcheck);
			IO_Set(PIN_CORE_VID_0, 1); // (expected 1.0002V)
			IO_Set(PIN_SCPU_VID_0, 1); // (expected 1.0302V)
		//	IO_Set(PIN_SCPU_VID_0, 1);
		}
	}
	else
	{
		printf("\033[1;44middq done bit is 0 !!\033[0m\n");
		IO_Set(PIN_CORE_VID_0, 1); // (expected 1.0002V)
		IO_Set(PIN_SCPU_VID_0, 1); // (expected 1.0302V)
	//	IO_Set(PIN_SCPU_VID_0, 1);
	}
#else
	{
		printf("\033[1;44middq function not ready\033[0m\n");
		// do nothing...
	}
#endif

	return 0;
}
#endif //#ifdef CONFIG_SYS_VIDDQ
