#ifndef _GDMA_EXPORT_H_
#define _GDMA_EXPORT_H_

#include <linux/types.h>
#include "rtk_gdma_export_user.h"

/*How many phyaddress that gdma needs*/
#define MAX_SEG_NUM 3
#define MAX_OSD_NUM 3


#define GDMA_SUCCESS  0
#define GDMA_FAIL    -1
#define COMP_BUF_FULL -2
#define COMP_HW_USED -3
#define COMP_PICOBJ_INFO_ERROR -4
#define COMP_ALLOC_NOT_READY  -5
#define COMP_BUF_WORK_FULL -6
#define COMP_INT_NOT_READY -7

typedef enum _SEG_NUMBER {
	INIT_PIXEL_SEG = -1,
	ONE_PIXEL_SEG = 0,
	TWO_PIXEL_SEG,
	FOUR_PIXEL_SEG,
	NOT_DEFIEND_PIXEL_SEG
} SEG_NUMBER;

typedef enum {
	C0  = 0,
	C1,
	C2,
	C3,
} E_BLEND_ORDER;

typedef enum {
	VO_GRAPHIC_BLOCK,
	VO_GRAPHIC_SEQUENTIAL
} VO_GRAPHIC_STORAGE_MODE;

typedef enum {
	VO_OSD_COLOR_RGB,
	VO_OSD_COLOR_BGR,
	VO_OSD_COLOR_GRB,
	VO_OSD_COLOR_GBR,
	VO_OSD_COLOR_RBG,
	VO_OSD_COLOR_BRG
} VO_OSD_RGB_ORDER;

typedef enum {
	GDMA_PLANE_OSD1 = 1,
	GDMA_PLANE_OSD2 = 2,
	GDMA_PLANE_OSD3 = 3,
	GDMA_PLANE_MAXNUM = 4,
	GDMA_PLANE_NONE = -1
} GDMA_DISPLAY_PLANE;

typedef struct {
	short x;
	short y;
	u_short width;
	u_short height;
} VO_RECTANGLE;

/*
 *
 * syncstamp  : syncstamp number
 * matchNum   : how many osd plane has the same syncstamp picture.
 * matchPlane : indicate which osd plane has the same syncstamp picture.
 *
*/
typedef struct {
	u64 syncstamp;
	uint matchNum;
	uint matchPlane[GDMA_PLANE_MAXNUM];
} GDMA_PIC_MATCH_SYNCSTAMP;

typedef struct {
	SOURCE_TYPE src_type;
	VO_OSD_COLOR_FORMAT format;
	FBDC_COLOR_FORMAT fbdc_format;
	unsigned int context;
	unsigned int key_en;
	int colorkey;
	int alpha;
	unsigned int x;
	unsigned int y;
	unsigned int width;
	unsigned int height;
	unsigned int address;
	unsigned int pitch;
	unsigned int seg_addr[MAX_SEG_NUM];
	INBAND_CMD_GRAPHIC_3D_MODE picLayout;
	unsigned int compressed;
	SEG_NUMBER seg_num;
	VO_GRAPHIC_PLANE plane;
	int dst_x;
	int dst_y;
	int dst_width;
	int dst_height;
	int layer_used;
	osd_planealpha_ar plane_ar;
	osd_planealpha_gb plane_gb;
	osd_clear_x clear_x;
	osd_clear_y clear_y;
	int show;
	unsigned int paletteformat; /* YVUA-->0  BGRA-->1 */
	u_char paletteIndex;
	VO_OSD_RGB_ORDER rgb_order;
	GDMA_PIC_MATCH_SYNCSTAMP syncInfo;
	int scale_factor;	/* layer scale factor based on panel size & fb size */
	int isFromOffline;					/* image data from offline */
} GDMA_PICTURE_OBJECT;

/*
*	for each OSD HW
*/
typedef struct {
	int fbdc_num;
	int normal_num;
	E_BLEND_ORDER onlineOrder;          /*  for final display use */
	GDMA_PICTURE_OBJECT fbdc[3];        /*  for FBDC */
	GDMA_PICTURE_OBJECT normal[1];      /*  for Mouse or Normal OSD */
} COMPOSE_PICTURE_OBJECT;


typedef struct {
	int layer_num;  /* indicate how many graphic layer (OSD HW) after GDMA driver arranged */
	COMPOSE_PICTURE_OBJECT layer[GDMA_PLANE_MAXNUM];
	int cookie;
} PICTURE_LAYERS_OBJECT;

void getDispSize(VO_RECTANGLE *disp_rec);
int GDMA_OSD1OnTop              (bool);
int GDMA_ConfigVFlip            (bool);
int GDMA_ReceivePicture         (PICTURE_LAYERS_OBJECT *);
int GDMA_ConfigInputSrc_4k2k (bool);
int GDMA_ConfigOSDEnable (bool);
int GDMA_ConfigOSDxEnable (GDMA_DISPLAY_PLANE disPlane, bool flag);

#if 0
void fwif_color_set_scaleosd_sharpness_driver(unsigned char src_index, unsigned char scale_factor);
#endif
#endif
