#include <command.h>
#include <common.h>
#include <drivers/mcp/rtk_mcp_scpu.h>
//#include <asm/arch/rbus/otp_reg.h>
#include <asm/arch/factorylib_ro.h>
#include <asm/arch/system.h>
#include <malloc.h>
#include <burn.h>
#ifdef CONFIG_SECURE_STORE_KEY_RESERVE
#include <asm/arch/ss_key_reserve.h>
#include <tar.h>
#endif

typedef enum {
	HDCP_14 = 14,
    HDCP_21 = 21,
    HDCP_22 = 22,
    HDCP_MERGE_14 = 361,
    HDCP_MERGE_22 = 362,
} HDCP_KEY_VERSION;

#define _DEBUG_CMD_BURN

extern int do_verify_read_facro (char *image_name, unsigned int image_addr,unsigned int *image_size);
extern int burn_secureKey(unsigned int key_addr, unsigned int key_size, char * key_name);
extern int delete_secureKey(char *key_name);
extern int list_secureKey(char *key_name);

#if defined(_DEBUG_CMD_BURN)
void hexdump(const void *buf, unsigned int length)
{
	unsigned int i;
	unsigned char *ptr = (unsigned char *)buf;

	if ((buf == NULL) || (length == 0))
	{
		printf("NULL\n");
		return;
	}

	printf("===> %s(0x%x, 0x%x):\n", __FUNCTION__, (unsigned int)buf, length);
	for (i = 0; i < length; i++)
	{
		printf("%02X ", ptr[i]);

		if ((i & 0xf) == 0xf)
		{
			printf("\n");
		}
	}
	printf("\n");
}
#endif

void clear_buffer(void *p,unsigned int s)
{
	memset(p, 0 , s);
	flush_cache((unsigned long)p, s);
	return;
}

unsigned int burn_cal_checksum( unsigned int start_address, unsigned int len )
{
	unsigned int i, checksum = 0;
	unsigned char * pcBuf = (unsigned char *) start_address;
	for( i=0; i<len; i++ )
	{
		checksum += pcBuf[i];
	}
	return checksum;
}

unsigned int burn_get_checksum( unsigned int checksum_address )
{
	unsigned int checksum_ref = 0;
	unsigned char * pcBuf = (unsigned char *) checksum_address;
	checksum_ref = pcBuf[0];
	checksum_ref = (checksum_ref<<8) | pcBuf[1];
	checksum_ref = (checksum_ref<<8) | pcBuf[2];
	checksum_ref = (checksum_ref<<8) | pcBuf[3];
	return checksum_ref;
}

unsigned int image_C_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x00200000;

int receive_hdcp_merged(void)
{
	char * cmd;
	unsigned int image_size = 0;
	unsigned int input_size = 376+1016;
    unsigned int checksum, checksum_ref;

	int rcode = CMD_RET_SUCCESS;
	cmd = (char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd, 0, SRING_SIZE);

	//1. download image file to image_A_addr
	snprintf(cmd, sizeof(cmd), "loady %x", image_C_addr);
	printf("%s...\n", cmd);
	rcode = run_command(cmd, 0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	image_size = getenv_ulong("filesize", 16, 0);
	
	if (image_size != input_size)
	{
		if( image_size == (input_size+4) ) {
            checksum = burn_cal_checksum(image_C_addr, input_size);
            checksum_ref = burn_get_checksum(image_C_addr + input_size);
            
		    if( checksum != checksum_ref ) {
			    printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			    printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			    printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			    goto FAIL;
		    }            
		}
        else{
            printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, input_size);
		    goto FAIL;
        }
	}

    return CMD_RET_SUCCESS;
FAIL:
	return CMD_RET_FAILURE;

}

int do_burn_hdcp_mac3_general(const unsigned int hdcp_ver);
int do_burn_hdcp_14(void)
{
	return do_burn_hdcp_mac3_general(HDCP_14);
}

int do_burn_hdcp_21(void)
{
	return do_burn_hdcp_mac3_general(HDCP_21);
}

int do_burn_hdcp_22 (void)
{
	return do_burn_hdcp_mac3_general(HDCP_22);
}

int do_burn_hdcp_merged (void)
{
    
	if(receive_hdcp_merged() == CMD_RET_FAILURE)
	{
        return CMD_RET_FAILURE;
    }

	return ( do_burn_hdcp_mac3_general(HDCP_MERGE_14) || do_burn_hdcp_mac3_general(HDCP_MERGE_22));
}

int do_burn_hdcp_mac3_general(const unsigned int hdcp_ver)
{
	printf("new\n");
	char * cmd = NULL;
	char * image_name = FACTORY_RO_HDCP_2_2_BIN;
	int rcode = CMD_RET_SUCCESS;
//	unsigned int Kh_key[4];
//	unsigned int Kh_key_devID[4];
	unsigned int image_size;
	unsigned int input_size;
	unsigned int write_size;
	unsigned int image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	unsigned int image_B_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x00100000;
	unsigned int checksum, checksum_ref;
	unsigned int do_checksum_verify;

	if (hdcp_ver == HDCP_14  || hdcp_ver == HDCP_MERGE_14 ) {
		image_name = FACTORY_RO_HDCP_1_4_BIN;
		input_size = 376; // get 376
		write_size = 325; //only write first 325
	}
	else if (hdcp_ver == HDCP_21 ) {
		image_name = FACTORY_RO_HDCP_2_1_BIN;
		input_size = 1016; // 1016
		write_size = 1016;
	}
	else if (hdcp_ver == HDCP_22 || hdcp_ver == HDCP_MERGE_22 ) {
		image_name = FACTORY_RO_HDCP_2_2_BIN;
		input_size = 1016; // 1016
		write_size = 1016;
	}	
    else {
		// not supported version
		goto FAIL;
	}

	cmd = (char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd, 0, SRING_SIZE);

	do_checksum_verify = 0;

	if( hdcp_ver == HDCP_MERGE_14 )
	{
		memcpy((void *)image_A_addr, (const void *)image_C_addr, input_size);
		image_size = write_size;
	}
	else if(hdcp_ver == HDCP_MERGE_22)
	{
		memcpy((void *)image_A_addr, (const void *)(image_C_addr +  376), input_size);
		image_size = write_size;
	}
	else
	{
		//1. download image file to image_A_addr
		snprintf(cmd, sizeof(cmd), "loady %x", image_A_addr);
		printf("%s...\n", cmd);
		rcode = run_command(cmd, 0);
		if (rcode != 0)
		{
			printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
			goto FAIL;
		}
		image_size = getenv_ulong("filesize", 16, 0);

		if (hdcp_ver == HDCP_14){
			if((image_size == 336) || (image_size == (336 + 4))){
				input_size = 336; // sharp hdcp1.4 is 336
				write_size = 336; 			
			}
		}

		if (hdcp_ver == HDCP_22){
			if((image_size == 896) || (image_size == (896 + 4))){
				input_size = 896; // sharp hdcp2.2 is 896
				write_size = 896; 			
			}			
		}
		
		if (image_size != input_size)
		{
			if( image_size == (input_size+4) ) {
				do_checksum_verify = 1;
			}
			else {
				printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, input_size);
				goto FAIL;
			}
		}

	}


	//	  copy image_A_addr to image_B_addr
	memcpy((void *)image_B_addr, (const void *)image_A_addr, image_size);

	image_size = input_size; // [ 325, 1016, 1016 ]

	//2.2 verify checksum
	checksum = burn_cal_checksum(image_B_addr, input_size);
	printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);

	if( do_checksum_verify ) {
		checksum_ref = burn_get_checksum(image_B_addr + input_size);
		if( checksum != checksum_ref ) {
			printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			goto FAIL;
		}
	}
	else {
		checksum_ref = checksum; // veirfy later
	}

	if(input_size != write_size){
		image_size = write_size;
		checksum = burn_cal_checksum(image_B_addr, image_size);			
		checksum_ref = checksum;
	}	
	printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);

	//2.3 copy image_B_addr to image_A_addr
	memcpy((void *)image_A_addr, (const void *)image_B_addr, image_size);

	//5. write image file from DDR to facro section
	snprintf(cmd, sizeof(cmd),"facro write %s %x  %x;facro save", image_name, image_A_addr, write_size);
	printf("%s...\n", cmd);
	rcode = run_command(cmd,0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}

	memset((char *)image_A_addr, 0x16, 1024);
	memset((char *)image_B_addr, 0x17, 1024);

	//6. read image file from facro section and compare
	printf("read %s to 0x%08x...\n", image_name, image_B_addr);
	rcode = do_verify_read_facro(image_name, image_B_addr, &image_size);
	if (rcode)
	{
		printf("[INFO %s] %s %s: failed\n", __FUNCTION__,image_name,"file not found");
		goto FAIL;
	}
	printf("reading size is %d\n", image_size);

	//9 caculate checksum again
	checksum = burn_cal_checksum( image_B_addr, write_size);
	if( checksum != checksum_ref ) {
		printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
		printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
		printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
		goto FAIL;
	}

	if (cmd)
	{
		free(cmd);
	}

	if (hdcp_ver == HDCP_14 || hdcp_ver == HDCP_MERGE_14 ) {
		printf("save hdcp1.4 success\n");
	}
	else if(hdcp_ver == HDCP_21) {
		printf("save hdcp2.1 success\n");
	}
	else if(hdcp_ver == HDCP_22  || hdcp_ver == HDCP_MERGE_22) {
		printf("save hdcp2.2 success\n");
	}
	return CMD_RET_SUCCESS;

FAIL:

	if (cmd)
	{
		free(cmd);
	}

	if (hdcp_ver == HDCP_14) {
		printf("save hdcp1.4 fail\n");
	}
	else if(hdcp_ver == HDCP_21) {
		printf("save hdcp2.1 fail\n");
	}
	else if(hdcp_ver == HDCP_22) {
		printf("save hdcp2.2 fail\n");
	}
	return CMD_RET_FAILURE;
}

int do_burn_mac ( char * mac_address )
{
	char * cmd;
	char * image_name;
	int rcode = CMD_RET_SUCCESS;
	unsigned int image_size;
	unsigned int image_size2;
	unsigned int image_A_addr;
	unsigned int image_B_addr;
	unsigned int checksum, checksum_ref;

	image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	image_B_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR+0x00100000;

	cmd=(char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd,0,SRING_SIZE);

	//1. get mac address from uart
	if( mac_address == NULL ) {
		snprintf(cmd, sizeof(cmd),"loady %x", image_A_addr);
		PRINT_DEBUG(cmd);
		PRINT_DEBUG("\n");
		rcode=run_command(cmd,0);
		if (rcode != 0)
		{
			printf("[INFO %s] %s failed\n", __FUNCTION__, cmd);
			goto FAIL;
		}
		image_size = getenv_ulong("filesize", 16, 0);
		if (image_size==0)
		{
			printf("[INFO %s] %s failed. image_size is zero\n", __FUNCTION__, cmd);
			goto FAIL;
		}
	}
	else {
		image_size = strlen(mac_address);//strlen("xx.xx.xx.xx.xx.xx");
		printf("str len \"%s\" is %d\n", mac_address, strlen(mac_address) );
		if( 17 == image_size ) {
			memcpy( (void *)image_A_addr, mac_address, image_size );
		}
		else if( 12 == image_size ) {
			// do nothing
			memset( (void *)image_A_addr, 0x2E, 17);
			memcpy( (void *)image_A_addr,    mac_address,    2 );
			memcpy( (void *)(image_A_addr+3),  mac_address+2,  2 );
			memcpy( (void *)(image_A_addr+6),  mac_address+4,  2 );
			memcpy( (void *)(image_A_addr+9),  mac_address+6,  2 );
			memcpy( (void *)(image_A_addr+12), mac_address+8,  2 );
			memcpy( (void *)(image_A_addr+15), mac_address+10, 2 );
			image_size = 17;
		}
		else {
			printf("[INFO %s] input mac string error, expected %d bytes\n", __FUNCTION__, image_size);
			goto FAIL;
		}
	}

	checksum_ref = burn_cal_checksum( image_A_addr, image_size );

	//2. write image file from DDR to facro section
	image_name = FACTORY_RO_MAC_BIN;
	snprintf(cmd, sizeof(cmd),"facro write %s %x  %x;facro save", image_name, image_A_addr, image_size);
	printf("%s...\n", cmd);
	rcode=run_command(cmd,0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	//3. read and check
	memset((void *)image_B_addr, 0, 32);
	rcode = do_verify_read_facro(image_name, image_B_addr, &image_size2);
	if (rcode)
	{
		printf("[INFO %s] file %s not found\n", __FUNCTION__, image_name);
		goto FAIL;
	}
	if( image_size2 != image_size ) {
		printf("[INFO %s] file size not match\n", __FUNCTION__);
		goto FAIL;
	}
	printf("reading size is %d\n", image_size);

	checksum = burn_cal_checksum( image_B_addr, image_size );

	if( checksum != checksum_ref ) {
		printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
		printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
		printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
		goto FAIL;
	}

	if (cmd)
	{
		free(cmd);
	}
	printf("save mac success\n");
	return CMD_RET_SUCCESS;

FAIL:
	if (cmd)
	{
		free(cmd);
	}
	printf("save mac fail\n");
	return CMD_RET_FAILURE;
}

#ifdef CONFIG_SECURE_STORE_KEY_RESERVE
int do_burn_widevine (void)
{
	char * cmd;
	char * image_name;
	int rcode = CMD_RET_SUCCESS;
	unsigned int image_size = 0;
	unsigned int input_size = 304;
	unsigned int write_size = 304;
	unsigned int image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	unsigned int image_B_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR+0x00100000;
	unsigned int checksum, checksum_ref;
	unsigned int do_checksum_verify = 0;

	cmd=(char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd,0,SRING_SIZE);

	image_name = FACTORY_RO_WIDEVINE_BIN;

	//1. get widevine file
	snprintf(cmd, sizeof(cmd),"loady %x", image_A_addr);
	printf("%s...\n",cmd);
	rcode=run_command(cmd,0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	image_size = getenv_ulong("filesize", 16, 0);
	if (image_size==0)
	{
		printf("[INFO %s] %s failed. image_size is zero\n", __FUNCTION__, cmd);
		goto FAIL;
	}

	if (image_size != input_size)
	{
		if( image_size == (input_size+4) ) {
			do_checksum_verify = 1;
		}
		else {
			printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, input_size);
			goto FAIL;
		}
	}

	memcpy((void *)image_B_addr, (const void *)image_A_addr, image_size);
	
	checksum = burn_cal_checksum( image_B_addr, input_size );
	if(do_checksum_verify){
		checksum_ref = burn_get_checksum( image_B_addr + input_size );
		if( checksum != checksum_ref ) {
			printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			goto FAIL;
		}
	}
	else{
		checksum_ref = checksum;
	}

	snprintf(cmd, sizeof(cmd),"facro write %s %x  %x;facro save", image_name, image_A_addr, write_size);
	printf("%s...\n", cmd);
	rcode = run_command(cmd,0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}

	memset((char *)image_B_addr, 0, 1024);

	printf("read %s to 0x%08x...\n", image_name, image_B_addr);
	rcode = do_verify_read_facro(image_name, image_B_addr, &image_size);
	if (rcode)
	{
		printf("[INFO %s] %s %s: failed\n", __FUNCTION__,image_name,"file not found");
		goto FAIL;
	}
	printf("reading size is %d\n", image_size);

	//9 caculate checksum again
	checksum = burn_cal_checksum( image_B_addr, write_size);
	if( checksum != checksum_ref ) {
		printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
		printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
		printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
		goto FAIL;
	}

	if (cmd)
	{
		free(cmd);
	}
	printf("save widevine success\n");
	return CMD_RET_SUCCESS;

FAIL:
	if (cmd)
	{
		free(cmd);
	}
	printf("save widevine fail\n");
	return CMD_RET_FAILURE;

}

int do_burn_ESN(void)
{
	char * cmd = NULL;
	int rcode = CMD_RET_SUCCESS;
	char * image_name = SS_KEY_RES_ESN_BIN;
	unsigned int image_size;
	unsigned int input_size = 304;
	unsigned int write_size = 304;
	unsigned int image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	unsigned int image_B_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x00100000;
	unsigned int checksum, checksum_ref;
	unsigned int do_checksum_verify;
	unsigned int ret = 0;
	ss_key_res_info* p;

	cmd = (char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd, 0, SRING_SIZE);

	do_checksum_verify = 0;

	//1. download image file to image_A_addr
	snprintf(cmd, sizeof(cmd), "loady %x", image_A_addr);
	printf("%s...\n", cmd);
	rcode = run_command(cmd, 0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	image_size = getenv_ulong("filesize", 16, 0);
	if (image_size != input_size)
	{
		if( image_size == (input_size+4) ) {
			do_checksum_verify = 1;
		}
		else {
			printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, input_size);
			goto FAIL;
		}
	}
	
	//	  copy image_A_addr to image_B_addr
	memcpy((void *)image_B_addr, (const void *)image_A_addr, image_size);	
	
	if( do_checksum_verify ) {
		checksum = burn_cal_checksum(image_B_addr, input_size);
		checksum_ref = burn_get_checksum(image_B_addr + input_size);
		if( checksum != checksum_ref ) {
			printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			goto FAIL;
		}
	}
	memcpy((void *)image_A_addr, (const void *)image_B_addr, write_size);
	
	if(ss_key_res_init() < 0){
		printf("[INFO %s] ss_key_res_init fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_info_init(&p) < 0){
		printf("[INFO %s] ss_key_res_info_init fail!\n",__FUNCTION__);
		goto FAIL;		
	}
	
	if(ss_key_res_write(image_name,(char *)image_A_addr,write_size) < 0){
		printf("[INFO %s] ss_key_res_write fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_save() < 0){
		printf("[INFO %s] ss_key_res_save fail!\n",__FUNCTION__);
		goto FAIL;		
	}

	if(ss_key_res_reload() < 0){
		printf("[INFO %s] ss_key_res_reload fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_read(image_name,image_B_addr,&image_size) < 0){
		printf("[INFO %s] ss_key_res_write fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if((image_size != write_size) || ((ret = memcmp(image_A_addr,image_B_addr,write_size)) != 0)){
		printf("[INFO %s] verify fail image_size:%d write_size:%d ret:%d!\n",__FUNCTION__,image_size,write_size,ret);
		goto FAIL;
	}

	if(ss_key_res_info_burn(ESN) < 0){
		printf("[INFO %s] ss_key_res_info_burn fail!\n",__FUNCTION__);
		goto FAIL;	

	}
	if (cmd)
	{
		free(cmd);
	}

	printf("save ESN success\n");
	return CMD_RET_SUCCESS;
FAIL:
	if (cmd)
	{
		free(cmd);
	}
	
	printf("save ESN fail\n");
	return CMD_RET_FAILURE;
}

int do_burn_playready(void)
{
	char * cmd = NULL;
	int rcode = CMD_RET_SUCCESS;
	char * image_name = SS_KEY_RES_PLAYREADY_BIN;
	unsigned int image_size;
	unsigned int input_size;
	unsigned int write_size;
	unsigned int image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	unsigned int image_B_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x00100000;
	unsigned int checksum, checksum_ref;
	unsigned int do_checksum_verify;
	unsigned int ret = 0;
	unsigned char *ptr_A = image_A_addr;
	ss_key_res_info* p;

	cmd = (char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		goto FAIL;
	}
	memset(cmd, 0, SRING_SIZE);

	do_checksum_verify = 0;

	//1. download image file to image_A_addr
	snprintf(cmd, sizeof(cmd), "loady %x", image_A_addr);
	printf("%s...\n", cmd);
	rcode = run_command(cmd, 0);
	if (rcode != 0)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	
	image_size = getenv_ulong("filesize", 16, 0);
	input_size = ((ptr_A[0] << 8) | ptr_A[1]) + 16; //raw data size + size + size xor + model id + crc16
	write_size = input_size;

	if(image_size != input_size)
	{
		if( image_size == (input_size+4) ) {
			do_checksum_verify = 1;
		}
		else {
			printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, input_size);
			goto FAIL;
		}
	}
	
	//	  copy image_A_addr to image_B_addr
	memcpy((void *)image_B_addr, (const void *)image_A_addr, image_size);	
	

	checksum = burn_cal_checksum(image_B_addr, input_size);
	if( do_checksum_verify ) {
		checksum_ref = burn_get_checksum(image_B_addr + input_size);
		if( checksum != checksum_ref ) {
			printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			goto FAIL;
		}
	}

	
	memcpy((void *)image_A_addr, (const void *)image_B_addr, write_size);
	
	if(ss_key_res_init() < 0){
		printf("[INFO %s] ss_key_res_init fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_info_init(&p) < 0){
		printf("[INFO %s] ss_key_res_info_init fail!\n",__FUNCTION__);
		goto FAIL;		
	}

	if(ss_key_res_write(image_name,(char *)image_A_addr,write_size) < 0){
		printf("[INFO %s] ss_key_res_write fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_save() < 0){
		printf("[INFO %s] ss_key_res_save fail!\n",__FUNCTION__);
		goto FAIL;		
	}

	if(ss_key_res_reload() < 0){
		printf("[INFO %s] ss_key_res_reload fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if(ss_key_res_read(image_name,image_B_addr,&image_size) < 0){
		printf("[INFO %s] ss_key_res_write fail!\n",__FUNCTION__);
		goto FAIL;
	}

	if((image_size != write_size) || ((ret = memcmp(image_A_addr,image_B_addr,write_size)) != 0)){
		printf("[INFO %s] verify fail image_size:%d write_size:%d ret:%d!\n",__FUNCTION__,image_size,write_size,ret);
		goto FAIL;
	}

	if(ss_key_res_info_burn(PLAYREADY) < 0){
		printf("[INFO %s] ss_key_res_info_burn fail!\n",__FUNCTION__);
		goto FAIL;	

	}

	if (cmd)
	{
		free(cmd);
	}

	printf("save playready success\n");
	return CMD_RET_SUCCESS;
FAIL:
	if (cmd)
	{
		free(cmd);
	}
	
	printf("save playready fail\n");
	return CMD_RET_FAILURE;	
}

int do_del_ss_key_res(key_res_num num)
{
	ss_key_res_info* p;
	unsigned int ret = 0;
	char path[128];

	memset(path,0,sizeof(path));
	if(ss_key_res_info_init(&p) > 0){
		printf("[INFO %s] Not key in ss key res!\n",__FUNCTION__);
		ret = CMD_RET_SUCCESS;
		goto end;
	}

	if((ret = ss_key_res_init()) < 0){
		printf("[INFO %s] ss_key_res_init fail!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;
		goto end;
	}


	printf("[INFO %s] ss_key_res have key\n",__FUNCTION__);
	switch (num){
		case ESN:
			strcpy(path,SS_KEY_RES_ESN_BIN);
			break;
		case PLAYREADY:
			strcpy(path,SS_KEY_RES_PLAYREADY_BIN);
			break;
		default:
			printf("[INFO %s] key path not support\n",__FUNCTION__);
			break;
	}
	if(ss_key_res_delete(path) < 0){
		printf("[INFO %s] ss_key_res_delete fail!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;
		goto end;
	}
	if(ss_key_res_save() < 0){
		printf("[INFO %s] ss_key_res_save fail!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;
		goto end;	
	}

	if(ss_key_res_info_del(num) < 0){
		printf("[INFO %s] del ss key fail!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;
		goto end;
	}
	
	ret = CMD_RET_SUCCESS;
end:
	return ret;
}

int do_list_ss_key_res(key_res_num num)
{
	ss_key_res_info* p;
	unsigned int ret = 0;
	
	if(ss_key_res_info_init(&p) > 0){
		printf("[INFO %s] Not key in ss key res!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;
		goto end;
	}

	if(p->exist[num] == 1){
		printf("[INFO %s] key in ss key res exist!\n",__FUNCTION__);
		ret = CMD_RET_SUCCESS;		
	}
	else{
		printf("[INFO %s] key in ss key res not exist!\n",__FUNCTION__);
		ret = CMD_RET_FAILURE;			
	}
end:
	return ret;

}
#endif

int do_burn_secureKey(char * key_name)
{
	char * cmd = NULL;
	int rcode = CMD_RET_SUCCESS;
	unsigned int image_size = 0;
	unsigned int key_size = 0;
	unsigned int checksum = 0, checksum_ref = 0;
	unsigned int image_A_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	int ret = 0;
	int check_sum_verify = 1;
	int key_size_check = 0;

	if(key_name != NULL){
		if(!strcmp(key_name, "widevine")){
			printf("[%s][%d]burn %s \r\n", __func__, __LINE__, key_name);
			key_size = 304;
			key_size_check = 1;
		}
	}
	cmd = (char *)malloc(SRING_SIZE);
	if (!cmd)
	{
		printf("[INFO %s] malloc buffer fail!\n", __FUNCTION__);
		goto FAIL;
	}
	memset(cmd, 0, SRING_SIZE);

	//1. download image file to image_A_addr
	snprintf(cmd, sizeof(cmd), "loady %x", image_A_addr);
	printf("%s...\n", cmd);
	rcode = run_command(cmd, 0);
	if (rcode != CMD_RET_SUCCESS)
	{
		printf("[INFO %s] %s: failed\n", __FUNCTION__, cmd);
		goto FAIL;
	}
	
	image_size = getenv_ulong("filesize", 16, 0);

	if(key_size_check){
		unsigned int comp_size = 0;

		comp_size = check_sum_verify? (image_size -4): image_size;
		if(key_size != comp_size){
			printf("[INFO %s] %s: failed, image_size not %d\n", __FUNCTION__, cmd, key_size);
			goto FAIL;
		}
	}
	if(check_sum_verify){
		key_size = image_size -4;

		// check checksum
		checksum = burn_cal_checksum(image_A_addr, key_size);
		checksum_ref = burn_get_checksum(image_A_addr + key_size);
		if( checksum != checksum_ref ) {
			printf("[INFO %s] checksum = 0x%08x\n", __FUNCTION__, checksum);
			printf("[INFO %s] checksum_ref = 0x%08x\n", __FUNCTION__, checksum_ref);
			printf("[INFO %s] compare checksum failed\n", __FUNCTION__);
			goto FAIL;
		}
	}else{
		key_size = image_size;
	}

	// burn optee key to flash
	ret =  burn_secureKey(image_A_addr, key_size, key_name);
	if(ret < 0){
		printf("[INFO %s] burn optee key fail!\n", __FUNCTION__);
		goto FAIL;
	}
	
	if (cmd)
	{
		free(cmd);
	}

	if(key_name == NULL)
		printf("save OPTEE KEY success\n");
	else
		printf("save %s success\n", key_name);

	return CMD_RET_SUCCESS;
FAIL:
	if (cmd)
	{
		free(cmd);
	}
	
	if(key_name == NULL)
		printf("save OPTEE KEY fail\n");
	else
		printf("save %s fail\n", key_name);
	return CMD_RET_FAILURE;
}

int do_delete_secureKey(char * key_name)
{
	int ret = 0;

	if(key_name == NULL)
		return CMD_RET_FAILURE;

	ret = delete_secureKey(key_name);
	if(ret < 0){
		printf("deleted %s fail\n", key_name);
		return CMD_RET_FAILURE;
	}
		
	printf("deleted %s success\n", key_name);
	return CMD_RET_SUCCESS;
}

int do_list_secureKey(char * key_name)
{
	int ret = 0;

	if(key_name == NULL)
		return CMD_RET_FAILURE;

	ret = list_secureKey(key_name);
	if(ret < 0){
		printf("key %s not exist\n", key_name);
		return CMD_RET_FAILURE;
	}
		
	printf("key %s exist\n", key_name);

	return CMD_RET_SUCCESS;
}

int do_burn (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
#ifdef CONFIG_SECURE_STORE_KEY_RESERVE
            int ret = 0;
#endif
	if (argc >= 2)
	{
		if (strcmp(argv[1], "hdcp1.4") == 0)
		{
			return do_burn_hdcp_14();
		}
		else if (strcmp(argv[1], "hdcp2.1") == 0)
		{
			return do_burn_hdcp_21();
		}
		else if (strcmp(argv[1], "hdcp2.2") == 0)
		{
			return do_burn_hdcp_22();
		}
		else if (strcmp(argv[1], "hdcp_merged") == 0)
		{
			return do_burn_hdcp_merged();
		}
		else if (strcmp(argv[1], "mac") == 0)
		{
			if( argc >=3 ) {
				return do_burn_mac(argv[2]);
			}
			else {
				return do_burn_mac("2a.3a.4a.5a.6a.7a");
			}
		}
		else if(strcmp(argv[1], "secureKey") == 0)
		{
			return do_burn_secureKey(NULL);
		}
		else if(strcmp(argv[1], "widevine") == 0)
		{
			return do_burn_secureKey("widevine");
		}
		else if (strcmp(argv[1], "delete") == 0)
		{
			if(argc == 3)
				return do_delete_secureKey(argv[2]);
			else
				return CMD_RET_USAGE;
		}
		else if (strcmp(argv[1], "list") == 0)
		{
			if(argc == 3)
				return do_list_secureKey(argv[2]);
			else
				return CMD_RET_USAGE;
		}
		#ifdef CONFIG_SECURE_STORE_KEY_RESERVE
		else if (strcmp(argv[1], "ESN") == 0)
		{
			return do_burn_ESN();
		}
		else if (strcmp(argv[1], "playready") == 0)
		{
			return do_burn_playready();
		}
		else if (strcmp(argv[1], "widevine") == 0)
		{
			return do_burn_widevine();
		}
		else if (strcmp(argv[1], "delete") == 0)
		{
			if(strcmp(argv[2], "ESN") == 0)
			{
				if((ret = do_del_ss_key_res(ESN)) == CMD_RET_SUCCESS){
					printf("delete ESN success!\n");
				}
				else{
					printf("delete ESN fail!\n");
				}
				return ret;
			}
			else if(strcmp(argv[2], "playready") == 0)
			{
				if((ret = do_del_ss_key_res(PLAYREADY)) == CMD_RET_SUCCESS){
					printf("delete playready success!\n");
				}
				else{
					printf("delete playready fail!\n");
				}
				return ret;
			}

		}
		else if (strcmp(argv[1], "list") == 0)
		{
			if(strcmp(argv[2], "ESN") == 0)
			{
				if((ret = do_list_ss_key_res(ESN)) == CMD_RET_SUCCESS)
					printf("esn.bin exist!\n");
				return ret;
			}
			else if(strcmp(argv[2], "playready") == 0)
			{
				if((ret = do_list_ss_key_res(PLAYREADY)) == CMD_RET_SUCCESS)
					printf("playready.bin exist!\n");
				return ret;
			}
		}
		#endif
	}

	return CMD_RET_USAGE;
}

U_BOOT_CMD
(
	burn, 3, 1,   do_burn, //do not display usage,avoid customer abuse
	NULL/*"burn image to factory via uart by rtice"*/,
	NULL/*"argu\n"
	"burn hdcp1.4\n"
	"burn hdcp2.1\n"
	"burn hdcp2.2\n"
	"burn mac \n"*/
);

