/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 * panel select command
*/
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <asm/arch/system.h>
#include <tar.h>
#include <factorylib.h>
#include "exports.h"
#include "cmd_irda.h"


struct IRDA_PROTOCOL irda_proto_list[IRDA_NUM_REMOTER] = {
        {0,   "DefaultSetting",     0,    "none"},
        {1,   "nec",                1,    "NEC"},
        {2,   "rc5",                2,    "RC5"},
        {3,   "sharp",              3,    "SHARP"},
        {4,   "sony",               4,    "SONY"},
        {5,   "c03",                5,    "C03"},
        {6,   "rc6",                6,    "RC6"},
        {7,   "nec",                7,    "RAW_NEC"},
        {8,   "rca",                8,    "RCA"},
        {9,   "panasonic",          9,    "PANASONIC"},
        {10,  "konka",              10,   "KONKA"},
        {11,  "rc6",                11,   "RAW_RC6"},
        {12,  "toshiba",            12,   "TSB"},
        {13,  "toshiba",            13,   "RAW_TSB"},
        {14,  "rca",                14,   "RAW_RCA"},
        {15,  "c03",                15,   "RAW_C03"},
        {16,  "konka",              16,   "RAW_KONKA"},
        {17,  "sony",               17,   "RAW_SONY"},
        {18,  "hikeen",             1,    "NEC"},
        {19,  "aeha",               18,   "AEHA"},
};
#define IRDA_PROTO_LIST_SIZE (sizeof(irda_proto_list) /sizeof(irda_proto_list[0]))

extern int gets(char *s);
extern char* strtrim(char *s);


int do_irda_selection(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        char buf[128] ={0};
        int i         = 0;
        int select    = -1;
        char value[64]= {0};

        const char *oldselect = getenv("irda_select");
        if(oldselect != NULL)
                select = simple_strtoul(oldselect, NULL, 10);
        printf("===================================================\n");
        if(oldselect != NULL && select < IRDA_PROTO_LIST_SIZE && select >= 0) {
                printf("The current selection is [%d] remoter=%s, protocol=%s\n", select, 
                            irda_proto_list[select].remote_name, irda_proto_list[select].proto_name);
        } else{
                printf("You don't select any irda remoter and protocol\n");
        }
        printf("===================================================\n");
        printf("Please select the irda remoter and protocol you want:\n");
        for( i = 0; i < IRDA_PROTO_LIST_SIZE ; i++) {
            printf("[%d] %s(%s)\n", i, irda_proto_list[i].remote_name, irda_proto_list[i].proto_name);
        }
        printf("=> ");
        gets(buf);
        select = simple_strtoul((char *)strtrim(buf), NULL, 10);
        if(select < IRDA_PROTO_LIST_SIZE && select > 0) {
                printf("\n===================================================\n");
                printf("you have selected [%d]  remoter=%s, protocol=%s\n", select, irda_proto_list[select].remote_name, irda_proto_list[select].proto_name);
                printf("===================================================\n");
                setenv("irda_select", strim(buf));
                snprintf(value, 64, "%d-%s", irda_proto_list[select].proto_id, irda_proto_list[select].remote_name);
                setenv("irda_select_value", value);
                saveenv();
        } else if((select == 0)) {
                printf("\nuse irda default setting! \n");
                setenv("irda_select", "0");
                snprintf(value, 64, "%s(%s)", irda_proto_list[0].remote_name, irda_proto_list[0].proto_name);
                setenv("irda_select_value", value);
                saveenv();  
        }else {
                printf("\n [ERR] wrong input! \n");
        }

        return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
        irda, 3, 0, do_irda_selection,
        "irda - select irda protocol",
        "This command is used to irda protocol in uboot level"
        " "
);

