#include <common.h>
#include <command.h>
#include <asm/arch/io.h>
#include <asm/arch/system.h>

#define KCPU_WAIT_HW_SEM_REG			(0xb801a630)
#define KCPU_WAIT_JUMP_DUMMY_REG		(0xb8060124)
#define KCPU_NOTIFY_STATUS_DUMMY_REG		(0xb801a614)

/*********************** Pre-defined magic number ***********************/
// for specific operation
#define MAGIC_KCPU_OP_MASK			(0xcafe1000)
#define MAGIC_KCPU_OP_SETUP_NORMAL		(0xcafe1001)	/* boot normal */
#define MAGIC_KCPU_OP_SETUP_RESCUE		(0xcafe1002)	/* boot rescue */
#define MAGIC_KCPU_OP_GO_NORMAL			(0xcafe1010)	/* go normal linux */
#define MAGIC_KCPU_OP_GO_RESCUE			(0xcafe1011)	/* go rescue linux */
#define MAGIC_KCPU_OP_GO_RESCUE_SEU		(0xcafe1012)	/* go rescue linux for secure update */
#define MAGIC_KCPU_OP_DO_MEM_MTEST		(0xcafe1013)	/* perform a memory test */
#define MAGIC_KCPU_OP_SETUP_S_KERNEL		(0xcafe1014)	/* load scpu kernel to ddr */
#define MAGIC_KCPU_OP_SETUP_FW_IMAGE		(0xcafe1015)	/* load fw image to ddr */

// for status report
#define MAGIC_KCPU_ST_MASK			(0xcafe2000)
#define MAGIC_KCPU_ST_OK			(0xcafe2000)	/* status ok */
#define MAGIC_KCPU_ST_FAIL			(0xcafe2001)	/* status fail */

// kboot status report
#define MAGIC_KCPU_LOAD_OPTEE_FAIL			(0xcafe3000)	/* status load noraml and rescue optee fail */
#define MAGIC_KCPU_LOAD_NORMAL_OPTEE			(0xcafe3002)	/* status load normal optee ok */
#define MAGIC_KCPU_LOAD_RESCUE_OPTEE			(0xcafe3003)	/* status load normal optee fail, load rescue optee ok */

// for kboot report the verison info check status
#define MAGIC_FW_VERSION_INFO_CHECK_MASK	(0xcccc0000)
#define MAGIC_FW_VERSION_INFO_CHECK_OK		(0xcccc0000)	/*check pass*/
#define MAGIC_KERNEL_VERSION_INFO_CHECK_FAIL	(0xcccc0001)	/*check fail, kernel version info error*/
#define MAGIC_KCPU_VERSION_INFO_CHECK_FAIL	(0xcccc0002)	/*check fail, kcpu version info error*/
#define MAGIC_AUDIO_VERSION_INFO_CHECK_FAIL	(0xcccc0004)	/*check fail, audio version info error*/
#define MAGIC_VIDEO_VERSION_INFO_CHECK_FAIL	(0xcccc0008)	/*check fail, video version info error*/
#define MAGIC_AUDIO2_VERSION_INFO_CHECK_FAIL	(0xcccc0010)	/*check fail, audio2 version info error*/
#define MAGIC_VIDEO2_VERSION_INFO_CHECK_FAIL	(0xcccc0020)	/*check fail, video2 version info error*/


// for test only
#define MAGIC_KCPU_TEST_DECRYPT			(0x0001cafe)
#define MAGIC_KCPU_TEST_LZMA			(0x1001cafe)
/*********************** Pre-defined magic number end ***********************/

//#define KBOOT_TEST_KCPU_DECRYPT
//#define KBOOT_TEST_KCPU_LZMA

void release_kcpu_hw_semaphore(void)
{
	rtd_outl(KCPU_WAIT_HW_SEM_REG, 0x0);
}

uint wait_kcpu_done_status(void)
{
	uint regValue;
	int i = 0;

	while(1) {
		regValue = rtd_inl(KCPU_WAIT_JUMP_DUMMY_REG);

		if ((regValue & MAGIC_KCPU_ST_MASK) == MAGIC_KCPU_ST_MASK) {
			break;
		}

		// reduce rbus traffic
		for (i = 0;i < 0x2000;i++)
			;
	}

	return regValue;
}

#ifdef CONFIG_KCPU_LOAD_OPTEE
int kboot_load_fw_status(void)
{
        uint regValue;

        regValue = rtd_inl(KCPU_WAIT_JUMP_DUMMY_REG);

        if (regValue == MAGIC_KCPU_LOAD_OPTEE_FAIL) {
                return 0;
        }

        return 1;
}

int kboot_load_normal_optee_status(void)
{
	uint regValue;

 	regValue = rtd_inl(KCPU_WAIT_JUMP_DUMMY_REG);
	if(regValue == MAGIC_KCPU_LOAD_NORMAL_OPTEE)
		return 1;

	return 0;
}
#endif

int do_kboot_wait(void)
{
#ifndef CONFIG_KCPU_LOAD_OPTEE
	uint status;

	printf("wait kcpu done ...\n");

	status = wait_kcpu_done_status();

	if (status == MAGIC_KCPU_ST_OK) {
		printf("status: ok\n");
		return 0;
	}
	else {
		printf("status: failed\n");
		return 1;
	}
#else
	return 0;
#endif
}

#if defined(Config_USE_SVFL_FW_TABLE_TRUE) || defined(Config_Support_SECURE_BOOT_VER_CHK_Enable)
uint wait_kcpu_check_version_info_status(void)
{
	uint regValue;
	int i = 0;

	while(1) {
		regValue = rtd_inl(KCPU_NOTIFY_STATUS_DUMMY_REG);

		if ((regValue & MAGIC_FW_VERSION_INFO_CHECK_MASK) == MAGIC_FW_VERSION_INFO_CHECK_MASK) {
			break;
		}

		// reduce rbus traffic
		for (i = 0;i < 0x2000;i++)
			;
	}

	return regValue;
}

int do_kboot_wait_check_version_result(void)
{
	uint status;

	printf("wait kcpu check version info done ...\n");

	status = wait_kcpu_check_version_info_status();

	if (status == MAGIC_FW_VERSION_INFO_CHECK_OK) {
		printf("FW version info check pass...\n");
		return 0;
	}
	else {
		if(status == MAGIC_KERNEL_VERSION_INFO_CHECK_FAIL)
			printf("Scpu kernel version info check Fail!\n");
		else if(status == MAGIC_KCPU_VERSION_INFO_CHECK_FAIL)
			printf("Kcpu kernel version info check Fail!\n");
		else if(status == MAGIC_AUDIO_VERSION_INFO_CHECK_FAIL)
			printf("Audio FW version info check Fail!\n");
		else if(status == MAGIC_VIDEO_VERSION_INFO_CHECK_FAIL)
			printf("Video FW version info check Fail!\n");
		else if(status == MAGIC_AUDIO2_VERSION_INFO_CHECK_FAIL)
			printf("Audio2 FW version info check Fail!\n");
		else if(status == MAGIC_VIDEO2_VERSION_INFO_CHECK_FAIL)
			printf("Video2 FW version info check Fail!\n");
		else
			printf("Unknown...Fail!\n");

		//while(1);	// FW version info check fail, boot stop...

		return 1;
	}
}
#endif

int do_kboot_normal(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_SETUP_NORMAL);

	release_kcpu_hw_semaphore();

	return 0;
}

int do_kboot_scpu_kernel(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_SETUP_S_KERNEL);

	release_kcpu_hw_semaphore();

	return 0;
}

int do_kboot_fw_image(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_SETUP_FW_IMAGE);

	release_kcpu_hw_semaphore();

	return 0;
}

int do_kboot_rescue(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_SETUP_RESCUE);

	release_kcpu_hw_semaphore();

	return 0;
}

int do_kboot_go_normal(void)
{
	do_kboot_wait();
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_GO_NORMAL);

	do_kboot_wait();

	release_kcpu_hw_semaphore();

	return 0;
}

int do_kboot_go_rescue(int secure_update)
{
	if (secure_update)
		rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_GO_RESCUE_SEU);
	else
		rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_OP_GO_RESCUE);

	release_kcpu_hw_semaphore();

	return 0;
}

#ifdef KBOOT_TEST_KCPU_LZMA
int do_kboot_t_lzma(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_TEST_LZMA);

	release_kcpu_hw_semaphore();

	return 0;
}
#endif

#ifdef KBOOT_TEST_KCPU_DECRYPT
int do_kboot_t_decrypt(void)
{
	rtd_outl(KCPU_WAIT_JUMP_DUMMY_REG, MAGIC_KCPU_TEST_DECRYPT);

	release_kcpu_hw_semaphore();

	return 0;
}
#endif

#ifdef Config_Process_DTE_KBoot_TRUE
int prepare_share_cmdline(void)
{
	extern char share_cmdline[SHARE_CMDLINE_SIZE];
	char *p_share_cmdline = (char *)SHARE_CMDLINE_ADDR;

	memset(p_share_cmdline, 0x0, SHARE_CMDLINE_SIZE);
	memcpy(p_share_cmdline, share_cmdline, SHARE_CMDLINE_SIZE);

	printf("s_cmdline(%d) = \"%s\"\n", strlen(p_share_cmdline), p_share_cmdline);

	flush_dcache_all();

	return 0;
}
#endif //#ifdef Config_Process_DTE_KBoot_TRUE

int do_kboot(cmd_tbl_t * cmdtp, int flag, int argc, char * const argv[])
{
	int ret = CMD_RET_SUCCESS;
	int secure_update = 0;

	if (argc > 1) {
#ifdef Config_Process_DTE_KBoot_TRUE
		prepare_share_cmdline();
#endif //#ifdef Config_Process_DTE_KBoot_TRUE

		if (strcmp(argv[1], "normal") == 0) {
			if (do_kboot_normal() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
		else if (strcmp(argv[1], "s_kernel") == 0) {
			if (do_kboot_scpu_kernel() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
		else if (strcmp(argv[1], "fw_image") == 0) {
                        if (do_kboot_fw_image() != 0) {
                                ret = CMD_RET_FAILURE;
                        }
                }
		else if (strcmp(argv[1], "rescue") == 0) {
			if (do_kboot_rescue() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
		else if (strcmp(argv[1], "go_k") == 0) {
			if (do_kboot_go_normal() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
		else if (strcmp(argv[1], "go_r") == 0) {
			if ((argc > 2) && (strcmp(argv[2], "-tab") == 0)) {
				secure_update = 0;
			}
			else {
				secure_update = 1;
			}
			if (do_kboot_go_rescue(secure_update) != 0) {
				ret = CMD_RET_FAILURE;
			}
			else {
				printf("start k rescue. (s enter infinite loop)\n");

				while(1){
					; // enter infinite loop
				}
			}
		}
		else if (strcmp(argv[1], "wait") == 0) {
			if (do_kboot_wait() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
#if defined(Config_USE_SVFL_FW_TABLE_TRUE)  || defined(Config_Support_SECURE_BOOT_VER_CHK_Enable)
		else if (strcmp(argv[1], "wait_v") == 0) {
			if (do_kboot_wait_check_version_result() != 0) {
				printf("FW version info check fail, boot stop...");
				while(1);

				//never reach
				ret = CMD_RET_FAILURE;
			}
		}
#endif
#ifdef KBOOT_TEST_KCPU_LZMA
		else if (strcmp(argv[1], "t_lzma") == 0) {
			if (do_kboot_t_lzma() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
#endif
#ifdef KBOOT_TEST_KCPU_DECRYPT
		else if (strcmp(argv[1], "t_dec") == 0) {
			if (do_kboot_t_decrypt() != 0) {
				ret = CMD_RET_FAILURE;
			}
		}
#endif
		else {
			return CMD_RET_USAGE;
		}
	}
	else {
		return CMD_RET_USAGE;
	}

	return ret;
}

U_BOOT_CMD(
	kboot,	CONFIG_SYS_MAXARGS,	1,	do_kboot,
	"start kboot function",
	"[normal | rescue | go_k | go_r [-tab]"
#ifdef KBOOT_TEST_KCPU_LZMA
	" | t_lzma"
#endif
#ifdef KBOOT_TEST_KCPU_DECRYPT
	" | t_dec"
#endif
	"]\n"
	"   normal  - read normal linux from flash\n"
	"   rescue  - read rescue linux from flash\n"
	"   go_k    - enter normal linux\n"
	"   go_r    - enter rescue linux\n"
	"     -tab  - non-secure update\n"
	"   wait    - wait kboot status\n"
#if defined(Config_USE_SVFL_FW_TABLE_TRUE) || defined(Config_Support_SECURE_BOOT_VER_CHK_Enable)
	"   wait_v  - wait kboot check FW version info status\n"
#endif
#ifdef KBOOT_TEST_KCPU_LZMA
	"   t_lzma  - test lzma function\n"
#endif
#ifdef KBOOT_TEST_KCPU_DECRYPT
	"   t_dec   - test decrypt function\n"
#endif
);

