/***************************************************
*
*		tee_rpmb_part.c
*		kui_rong@apowertec.com
*
*****************************************************/
#include <config.h>
#include <common.h>
#include <command.h>
#include <malloc.h>

#include <mmc.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <drivers/mcp/rtk_mcp_scpu.h>

/*for read otp data*/
#include <drivers/otp/rtk_otp_util.h>
#include <drivers/otp/rtk_otp_reg.h>

#include "./tee_rpmb_part.h"

//#define _DEBUG_RPMB_FS_
#ifdef _DEBUG_RPMB_FS_
#define RPMB_MSG_DEBUG(fmt, args...) printf("[RPMB][DEBUG] " fmt, ## args)
#else
#define RPMB_MSG_DEBUG(fmt, args...)     /* empty macro */
#endif
#define RPMB_MSG_INFO(fmt, args...) printf("[RPMB][INFO] " fmt, ## args)


/*define: copy from optee_os/core/tee/tee_rpmb_fs.c */
#define RPMB_STORAGE_START_ADDRESS		0
#define RPMB_FS_FAT_START_ADDRESS			512
#define TEE_FS_KM_FEK_SIZE					16  /* bytes */
#define TEE_RPMB_FS_FILENAME_LENGTH		224

#define RPMB_FS_MAGIC                   0x52504D42
#define FS_VERSION                      2
#define N_ENTRIES							8
#define FILE_IS_ACTIVE                  (1u << 0)
#define FILE_IS_LAST_ENTRY              (1u << 1)

#define RPMB_SZ_DATA_SHIFT		8
#define RPMB_SZ_DATA			256

#define TEE_SVC_STORAGE_MAGIC 0x53534600

#define SWAP32(x)		((((x) & 0x000000ff) << 24) | \
				 (((x) & 0x0000ff00) <<  8) | \
				 (((x) & 0x00ff0000) >>  8) | \
				 (((x) & 0xff000000) >> 24)   \
				)

#define TEE_AES_BLOCK_SIZE	16


/* otp relative*/
#define OTP_ARM_TZ_enable_addr		0x18038048
#define OTP_ARM_TZ_enable_offset	(OTP_ARM_TZ_enable_addr - OTP_DDR_MAP_BASE)
#define OTP_ARM_TZ_enable_lsb		21
#define OTP_ARM_TZ_enable_msb		24

#define OTP_RTK_CHIP_ID_addr		0x18038030
#define OTP_RTK_CHIP_ID_offset		(OTP_RTK_CHIP_ID_addr - OTP_DDR_MAP_BASE)
#define OTP_RTK_CHIP_ID_len			0x10

/**
 * File entry for a single file in a RPMB_FS partition.
 */
struct rpmb_fat_entry {
	uint32_t start_address;
	uint32_t data_size;
	uint32_t flags;
	uint32_t write_counter;
	uint8_t fek[TEE_FS_KM_FEK_SIZE];
	char filename[TEE_RPMB_FS_FILENAME_LENGTH];
};

/**
 * RPMB_FS partition data
 */
struct rpmb_fs_partition {
	uint32_t rpmb_fs_magic;
	uint32_t fs_version;
	uint32_t write_counter;
	uint32_t fat_start_address;
	/* Do not use reserved[] for other purpose than partition data. */
	uint8_t reserved[112];
};

/* Header of GP formated secure storage files */
struct tee_svc_storage_head {
	uint32_t magic;
	uint32_t head_size;
	uint32_t meta_size;
	uint32_t ds_size;
	uint32_t keySize;
	uint32_t maxKeySize;
	uint32_t objectUsage;
	uint32_t objectType;
	uint32_t have_attrs;
};

extern unsigned int mcp_dscpt_addr;


static unsigned int align_to_boundary(unsigned int size, unsigned int block_size)
{
	return ((size % block_size) ? (size / block_size + 1)*block_size : (size / block_size)*block_size);
}

static int read_data_from_rpmb(unsigned int addr, unsigned int size, void *buff)
{
	unsigned int blk = 0;
	unsigned int cnt = 0;
	char cmd[128] = {0};
	int rcode = CMD_RET_SUCCESS;

	if((buff == NULL) || (size == 0)){
		RPMB_MSG_INFO("[%s][%d] Parameter setting error!\n", __func__,__LINE__);
		return -1;
	}

	if((addr % RPMB_SZ_DATA) || (size % RPMB_SZ_DATA)){
		RPMB_MSG_INFO("[%s][%d] address or size not block aligned!\n",__func__,__LINE__);
		return -1;
	}

	blk = addr >> RPMB_SZ_DATA_SHIFT;
	cnt = size >> RPMB_SZ_DATA_SHIFT;

	snprintf(cmd, sizeof(cmd),"mmc rpmb read %x %x %x", (unsigned int)buff, blk, cnt);
	rcode=run_command(cmd,0);
	if (rcode != 0)
	{
		RPMB_MSG_INFO("[%s][%d] %s: failed!\n", __func__, __LINE__, cmd);
		return -1;
	}

	return rcode;
}

#ifdef _DEBUG_RPMB_FS_
static void RPMB_hexdump(const char * msg, const void *buf, unsigned int length)
{
	unsigned int i;
	unsigned char *ptr = (unsigned char *)buf;

	if ((buf == NULL) || (length == 0))
	{
		printf("NULL\n");
		return;
	}
	
	printf("===> %s(0x%x, 0x%x):\n", msg, (unsigned int)buf, length);
	for (i = 0; i < length; i++)
	{
		printf("%02X ", ptr[i]);

		if ((i & 0xf) == 0xf)
		{
			printf("\n");
		}
	}
	printf("\n");
}
#else
static void RPMB_hexdump(const char * msg, const void *buf, unsigned int length){}
#endif

static bool is_zero(const uint8_t *buf, size_t size)
{
	size_t i;

	for (i = 0; i < size; i++)
		if (buf[i])
			return 0;
	return 1;
}

static bool is_dead(const unsigned int *buf, size_t size)
{
	size_t i;

	for (i = 0; i < size; i++)
		if (buf[i] == 0xdeaddead){
			RPMB_MSG_DEBUG("data is 0xdeaddead!\n");
			return 1;
		}
	return 0;
}

static void key_swap(unsigned int *key, unsigned int size)
{
	unsigned int i = 0;

	for(i = 0 ; i < size; i++){
		key[i] = SWAP32(key[i]);
	}

	return ;
}

static int get_iv(unsigned char *iv, unsigned int len, unsigned short dst_blk)
{
	int i = 0;
	
	if((iv == NULL) || (len != TEE_AES_BLOCK_SIZE)){
		RPMB_MSG_INFO("[%s][%d] Parameter setting error!\n", __func__,__LINE__);
		return -1;
	}

	iv[0] = dst_blk & 0xff;
	iv[1] = (dst_blk >> 8) & 0xff;

	for(i = 2; i < TEE_AES_BLOCK_SIZE; i++)
		iv[i] = iv[i-1] ^ iv[i-2];

	return 0;
}

static int get_fek(unsigned char *fek, unsigned int len)
{
	int ret = 0;
	unsigned char ch_key[OTP_RTK_CHIP_ID_len];
	unsigned long tz_enable = 0;
	
	if((fek == NULL) || (len != TEE_FS_KM_FEK_SIZE)){
		RPMB_MSG_INFO("[%s][%d] Parameter setting error!\n", __func__,__LINE__);
		return -1;
	}

	if(!mcp_dscpt_addr)
		mcp_dscpt_addr = (unsigned int)malloc(CP_DSCPT_POOL_SIZE);

#if 0		// dec by cw
	unsigned int chip_id_cw = 116;	// info from auxcode
	ret = AES_ECB_decrypt(fek, len, fek, (unsigned int *)&chip_id_cw);
	if(ret){
		RPMB_MSG_INFO("[%s][%d] decrypt fek fail!\n", __func__,__LINE__);
		return -1;
	}
#else
	// check enable TZ
#ifdef CONFIG_OTP_UTIL
	ret = otp_read_bits(OTP_ARM_TZ_enable_offset, OTP_ARM_TZ_enable_msb, OTP_ARM_TZ_enable_lsb,&tz_enable);
#endif
	if((ret < 0)|| !(tz_enable & 0x6)){
		RPMB_MSG_INFO("[%s][%d] OTP_ARM_TZ_enable is not enable status!\n", __func__,__LINE__);
		return -1;
	}

	// get chip id
#ifdef CONFIG_OTP_UTIL
	ret = otp_read_region_byte(OTP_RTK_CHIP_ID_offset, ch_key, OTP_RTK_CHIP_ID_len);
#endif
	if((ret < 0) || is_dead((unsigned int *)ch_key, OTP_RTK_CHIP_ID_len/sizeof(unsigned int))){
		RPMB_MSG_INFO("[%s][%d] get chip id fail!\n", __func__,__LINE__);
		RPMB_hexdump("chip id", ch_key, OTP_RTK_CHIP_ID_len);
		return -1;
	}

	key_swap((unsigned int *)ch_key, OTP_RTK_CHIP_ID_len/sizeof(unsigned int));

	//RPMB_hexdump("chip id", ch_key, OTP_RTK_CHIP_ID_len);

#ifdef CONFIG_SYS_MCP_UTILITY
	ret = AES_ECB_decrypt(fek, len, fek,(unsigned int*)ch_key);
#else
	ret = -1;
#endif	
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d] decrypt fek fail!\n", __func__,__LINE__);
		return -1;
	}

#endif
	return 0;
}

static int optee_rpmb_fs_partition_data_check(unsigned int *fat_addr)
{
	struct rpmb_fs_partition partition_data;
	unsigned int size = 0;
	unsigned int addr = 0;
	unsigned char *p_buff = NULL;
	int ret =0;

	if(fat_addr == NULL){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// read RPMB FS data
	addr = RPMB_STORAGE_START_ADDRESS;
	size = align_to_boundary(sizeof(struct rpmb_fs_partition), RPMB_SZ_DATA);
	
	p_buff = (unsigned char *)malloc(size);
	if(p_buff == NULL){
		RPMB_MSG_INFO("[%s][%d]Malloc buffer fail!\n", __func__, __LINE__);
		return -1;
	}

	ret = read_data_from_rpmb(addr, size, p_buff);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]Read rpmb FS data fail!\n", __func__, __LINE__);
		ret = -1;
		goto final;
	}

	memcpy(&partition_data, p_buff, sizeof(struct rpmb_fs_partition));

	// check magic number & version
	if((partition_data.rpmb_fs_magic != RPMB_FS_MAGIC) || (partition_data.fs_version != FS_VERSION)){
		RPMB_MSG_INFO("[%s][%d]Error RPMB Header!magic = 0x%x \n", __func__, __LINE__, partition_data.rpmb_fs_magic);
		RPMB_MSG_INFO("[%s][%d]Error RPMB Header!version = 0x%x \n", __func__, __LINE__, partition_data.fs_version);
		ret = -1;
		goto final;
	}

	*fat_addr = partition_data.fat_start_address;
	ret = 0;

final:
	if(p_buff)
		free(p_buff);

	return ret;
}

static int optee_rpmb_fs_fat_find(unsigned int fat_start_addr, 
					const char *target_file_name, struct rpmb_fat_entry *fat_entry)
{
	struct rpmb_fat_entry *fat_entries = NULL;
	unsigned int size = 0;
	unsigned int addr = 0;
	int ret = 0;
	int hit_flag = 0;
	int last_entry_found = 0;
	int i = 0;

	if((target_file_name == NULL) || (fat_entry == NULL)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// read rpmb partition FAT
	addr = fat_start_addr;
	size = N_ENTRIES * sizeof(struct rpmb_fat_entry);
	size = align_to_boundary(size, RPMB_SZ_DATA);
	
	fat_entries = (struct rpmb_fat_entry *)malloc(size);
	if (!fat_entries) {
		RPMB_MSG_INFO("[%s][%d]malloc buffer fail, malloc buffer size = 0x%x\n", __func__, __LINE__, size);
		return -1;
	}

	// search target file
	while (!last_entry_found && !hit_flag){
		// read 8*fat_entry
		ret = read_data_from_rpmb(addr, size, fat_entries);
		if(ret < 0){
			RPMB_MSG_INFO("[%s][%d]Read rpmb FS data fail!\n", __func__, __LINE__);
			ret = -1;
			goto final;
		}

		// parse rpmb partition FAT
		for (i = 0; i < N_ENTRIES; i++) {
			//  Look for an entry, matching filenames
			if((strcmp(target_file_name,fat_entries[i].filename) == 0) && (fat_entries[i].flags & FILE_IS_ACTIVE)){
				memcpy(fat_entry, &(fat_entries[i]), sizeof(struct rpmb_fat_entry));
				//RPMB_MSG_DEBUG("[%s][%d]file-%d, Aloha file addr = 0x%x size = 0x%x \n",
				//	__func__, __LINE__, i,fat_entry->start_address, fat_entry->data_size);
				//RPMB_MSG_DEBUG("[%s][%d]filename %s \n",__func__, __LINE__, fat_entry->filename);
				hit_flag = 1;
				break;
			}

			if ((fat_entries[i].flags & FILE_IS_LAST_ENTRY) != 0) {
				RPMB_MSG_DEBUG("[%s][%d]File is last entry, name %s \n",__func__, __LINE__, fat_entries[i].filename);
				last_entry_found = 1;
				break;
			}

			/* Move to next fat_entry. */
			addr += sizeof(struct rpmb_fat_entry);
		}
	} 

	if(hit_flag == 0){
		RPMB_MSG_INFO("[%s][%d]Can not find %s!\n", __func__, __LINE__, target_file_name);
		ret = -1;
		goto final;
	}

	ret = 0;
	
final:
	if(fat_entries)
		free(fat_entries);

	return ret;
}
#ifdef _DEBUG_RPMB_FS_
static int optee_rpmb_fs_list(unsigned int fat_start_addr, char *file_name_list[], unsigned int *file_num)
{
	struct rpmb_fat_entry *fat_entries = NULL;
	unsigned int size = 0;
	unsigned int addr = 0;
	int ret = 0;
	int last_entry_found = 0;
	int i = 0;
	unsigned int file_count = 0;
	char *tmp_ptr = NULL;

	if((file_name_list == NULL) || (file_num == NULL)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// read rpmb partition FAT
	addr = fat_start_addr;
	size = N_ENTRIES * sizeof(struct rpmb_fat_entry);
	size = align_to_boundary(size, RPMB_SZ_DATA);
	
	fat_entries = (struct rpmb_fat_entry *)malloc(size);
	if (!fat_entries) {
		RPMB_MSG_INFO("[%s][%d]malloc buffer fail, malloc buffer size = 0x%x\n", __func__, __LINE__, size);
		return -1;
	}

	tmp_ptr = (char *)file_name_list;
	// search target file
	while (!last_entry_found){
		// read 8*fat_entry
		ret = read_data_from_rpmb(addr, size, fat_entries);
		if(ret < 0){
			RPMB_MSG_INFO("[%s][%d]Read rpmb FS data fail!\n", __func__, __LINE__);
			ret = -1;
			goto final;
		}

		// parse rpmb partition FAT
		for (i = 0; i < N_ENTRIES; i++) {
			//	Look for an entry, matching filenames
			if(fat_entries[i].flags & FILE_IS_ACTIVE){
				memcpy(tmp_ptr, fat_entries[i].filename, TEE_RPMB_FS_FILENAME_LENGTH);
				tmp_ptr += TEE_RPMB_FS_FILENAME_LENGTH;
				file_count++;
			}

			if (fat_entries[i].flags & FILE_IS_LAST_ENTRY) {
				last_entry_found = 1;
				break;
			}

			/* Move to next fat_entry. */
			addr += sizeof(struct rpmb_fat_entry);
		}
	} 

	*file_num = file_count;
	ret = 0;
	
final:
	if(fat_entries)
		free(fat_entries);

	return ret;
}
#endif
static int optee_rpmb_fs_block_decrypt(unsigned int rpmb_blk, unsigned char *fek, char *data_buf, unsigned int data_size)
{
	unsigned char iv[TEE_AES_BLOCK_SIZE] ;
	int ret = 0;
	//unsigned int addr = 0;
	
	if((data_buf == NULL)||(data_size != RPMB_SZ_DATA) || (fek == NULL)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;		
	}

	// get iv key
	ret = get_iv(iv, sizeof(iv), rpmb_blk);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]Get iv key fail!\n", __func__, __LINE__);
		return -1;
	}
	key_swap((unsigned int *)iv, sizeof(iv)/sizeof(unsigned int));
	
	//RPMB_hexdump("iv key", iv, sizeof(iv));

#ifdef CONFIG_SYS_MCP_UTILITY
	// dec data
	ret = AES_CBC_decrypt((unsigned char *)data_buf, data_size, (unsigned char*)data_buf, (unsigned int*)(fek), (unsigned int *)iv);
#else
	ret = -1;
#endif
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]dec data fail!\n", __func__, __LINE__);
		return -1;
	}

	//RPMB_hexdump("aloha dec data", data_buf, data_size);

	return 0;
}

static int optee_rpmb_fs_file_extract(struct rpmb_fat_entry fat_entry, char *data_buf, unsigned int data_size)
{
	unsigned int size = 0;
	unsigned int addr = 0;
	int ret = 0;
	int is_enc = 0;

	if((data_buf == NULL) || (data_size == 0)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// read aloha file
	addr = fat_entry.start_address;
	size = align_to_boundary(fat_entry.data_size, RPMB_SZ_DATA);
	if(size > data_size){
		RPMB_MSG_INFO("[%s][%d]data buffer too small, data_size = 0x%x !\n", __func__, __LINE__,data_size);
		return -1;
	}
	
	ret = read_data_from_rpmb(addr, size, data_buf);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]Read file data fail!\n", __func__, __LINE__);
		return -1;
	}

	// decrypt file data
	is_enc = !(is_zero(fat_entry.fek, TEE_FS_KM_FEK_SIZE));
	if(is_enc){ 
		unsigned int i = 0;
		unsigned int loop = 0;
		
		// get fek
		ret = get_fek(fat_entry.fek, TEE_FS_KM_FEK_SIZE);
		if(ret < 0){
			RPMB_MSG_INFO("[%s][%d]Get fek key fail!\n", __func__, __LINE__);
			return -1;
		}
		key_swap((unsigned int *)fat_entry.fek, TEE_FS_KM_FEK_SIZE/sizeof(unsigned int));
		
		//RPMB_hexdump("fek dec", fat_entry.fek, TEE_FS_KM_FEK_SIZE);

		loop = size >> RPMB_SZ_DATA_SHIFT;
		for(i=0; i<loop; i++){
			ret = optee_rpmb_fs_block_decrypt((addr>>RPMB_SZ_DATA_SHIFT) + i,  fat_entry.fek, data_buf + i*RPMB_SZ_DATA, RPMB_SZ_DATA);
			if(ret < 0){
				RPMB_MSG_INFO("[%s][%d]decrypt block fail!\n", __func__, __LINE__);
				return -1;
			}
		}

	}

	return 0;
}

static int optee_rpmb_fs_GPheader_check(struct tee_svc_storage_head  gp_head)
{
	return (gp_head.magic != TEE_SVC_STORAGE_MAGIC)?(-1):(0);
}

static int optee_rpmb_fs_file_context_extract(char *file_buff, unsigned int file_size, 
											char *file_context, unsigned int file_context_size)
{
	struct tee_svc_storage_head  gp_head;
	int ret = 0;
	
	if((file_buff == NULL) || (file_size < sizeof(struct tee_svc_storage_head)) \
		|| (file_context==NULL) || (file_context_size == 0)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// GP header check
	memcpy(&gp_head, file_buff, sizeof(struct tee_svc_storage_head));

	ret = optee_rpmb_fs_GPheader_check(gp_head);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]File GP header check error!\n", __func__, __LINE__);
		return -1;
	}

	// sanity check
	if(file_size < ( gp_head.head_size + gp_head.meta_size + gp_head.ds_size)){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	if(file_context_size < gp_head.ds_size){
		RPMB_MSG_INFO("[%s][%d]file_context is too small!\n", __func__, __LINE__);
		return -1;
	}

	//get  file context
	/*
	*	optee os rpmb fs file layout:	
	*	GP head + meta data + file data
	*/	
	memcpy(file_context, (void *)((unsigned int)file_buff + gp_head.head_size + gp_head.meta_size), gp_head.ds_size);

	return 0;
}

static int optee_rpmb_fs_file_context_size(char *file_buff, unsigned int file_size, unsigned int *file_ctx_size)
{
	struct tee_svc_storage_head  gp_head;
	int ret = 0;
	
	if((file_buff == NULL) || (file_size < sizeof(struct tee_svc_storage_head)) \
		|| (file_ctx_size==NULL) ){
		RPMB_MSG_INFO("[%s][%d]argument error!\n", __func__, __LINE__);
		return -1;
	}

	// GP header check
	memcpy(&gp_head, file_buff, sizeof(struct tee_svc_storage_head));

	ret = optee_rpmb_fs_GPheader_check(gp_head);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]File GP header check error!\n", __func__, __LINE__);
		return -1;
	}

	*file_ctx_size = gp_head.ds_size;
	
	return 0;
}

int get_file_context_from_rpmb(const char *file_name, char *context_buf, unsigned int buf_size)
{
	int ret = 0;
	unsigned int fat_addr = 0;
	struct rpmb_fat_entry fat_entry;
	
	unsigned int file_size = 0;
	char * file_buf = NULL;
	unsigned int file_context_size = 0;
	
#ifndef CONFIG_SUPPORT_EMMC_RPMB
	RPMB_MSG_INFO("[%s][%d]Un-define CONFIG_SUPPORT_EMMC_RPMB\n", __func__,__LINE__);
	return -1;
#endif

	// read RPMB FS data & check
	ret = optee_rpmb_fs_partition_data_check(&fat_addr);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]optee rpmb fs partition_data check fail!\n", __func__,__LINE__);
		return -1;
	}

	// find file entry
	ret = optee_rpmb_fs_fat_find(fat_addr, file_name, &fat_entry);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]find aloha file fail!\n", __func__,__LINE__);
		return -1;
	}

	// alloc buffer for store file data
	file_size = align_to_boundary(fat_entry.data_size, RPMB_SZ_DATA);
	file_buf = (char *)malloc(file_size);
	if(file_buf == NULL){
		RPMB_MSG_INFO("[%s][%d]malloc aloha data buffer fail!\n", __func__,__LINE__);
		return -1;
	}

	//extract file data
	ret = optee_rpmb_fs_file_extract(fat_entry, file_buf, file_size);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]extract aloha file fail!\n", __func__,__LINE__);
		ret = -1;
		goto final;
	}

	// extract aloha file context size
	ret = optee_rpmb_fs_file_context_size(file_buf, file_size, &file_context_size);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]extract aloha file context size fail!\n", __func__,__LINE__);
		ret = -1;
		goto final;
	}

	if(file_context_size > buf_size){
		RPMB_MSG_INFO("[%s][%d]file context buffer too small!\n", __func__,__LINE__);
		ret = -1;
		goto final;
	}

	// extract aloha file context
	ret = optee_rpmb_fs_file_context_extract(file_buf, file_size, context_buf, file_context_size);
	if(ret < 0){
		RPMB_MSG_INFO("[%s][%d]extract aloha file context fail!\n", __func__,__LINE__);
		ret = -1;
		goto final;
	}

	ret = 0;
final:
	if(file_buf)
		free(file_buf);
	return ret;
}

#ifdef _DEBUG_RPMB_FS_
int do_tee_rpmb(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = CMD_RET_USAGE;

	if (strcmp(argv[1], "aloha") == 0){
		tps_fw_version_info p_version_info;
		ret = get_file_context_from_rpmb(TEE_ALOHA_PATH, &p_version_info, sizeof(tps_fw_version_info));
		if(ret < 0){
			RPMB_MSG_INFO("Get aloha from rpmb fail!\n");
			ret = CMD_RET_FAILURE;
		}else{
			ret = CMD_RET_SUCCESS;
			printf("\r\n");
			RPMB_MSG_INFO("%20s: 0x%x \n", "kernel_version", p_version_info.kernel_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "audio_version", p_version_info.audio_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "video_version", p_version_info.video_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "audio2_version", p_version_info.audio2_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "video2_version", p_version_info.video2_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "kcpu_version", p_version_info.kcpu_version);
			RPMB_MSG_INFO("%20s: 0x%x \n", "bootloader_version", p_version_info.bootloader_version);
		}
	}

	if (strcmp(argv[1], "list") == 0){
		char file_name_list[100][TEE_RPMB_FS_FILENAME_LENGTH];
		unsigned int file_num = 0;
		int i = 0;
		
		ret = optee_rpmb_fs_list(RPMB_FS_FAT_START_ADDRESS, file_name_list, &file_num);
		if(ret < 0){
			RPMB_MSG_INFO("Get file list fail!\n");
			ret = CMD_RET_FAILURE;
		}else{
			ret = CMD_RET_SUCCESS;
			printf("\r\n");
			RPMB_MSG_INFO("%10s: \r\n", "File List");
			for(i=0; i<file_num; i++)
				RPMB_MSG_INFO("%10s: %s \r\n", "  ", file_name_list[i]);
		}
	}

	if(strcmp(argv[1], "info") == 0){
		struct rpmb_fat_entry fat_entry;
		struct tee_svc_storage_head gp_head;
		int i = 0;
		char *tmp_buf = (char *)TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
		
		ret = optee_rpmb_fs_fat_find(RPMB_FS_FAT_START_ADDRESS, 
					argv[2], &fat_entry);
		if(ret < 0){
			RPMB_MSG_INFO("find file %s fail!\n", argv[2]);
			ret = CMD_RET_FAILURE;
			return ret;
		}

		ret = optee_rpmb_fs_file_extract(fat_entry, tmp_buf, align_to_boundary(fat_entry.data_size, RPMB_SZ_DATA));
		if(ret < 0){
			RPMB_MSG_INFO("[%s][%d]extract file %s file data fail!\n", __func__,__LINE__, argv[2]);
			ret = CMD_RET_FAILURE;
			return ret;
		}

		memcpy(&gp_head, tmp_buf, sizeof(struct tee_svc_storage_head));
		
		ret = CMD_RET_SUCCESS;
		printf("\r\n");
		RPMB_MSG_INFO("%20s: \r\n", "File Info");
		RPMB_MSG_INFO("%20s: %s \r\n", "File name" , fat_entry.filename);
		RPMB_MSG_INFO("%20s: 0x%x \r\n", "File size" , fat_entry.data_size);
		RPMB_MSG_INFO("%20s: 0x%x \r\n", "File addr" , fat_entry.start_address);
		RPMB_MSG_INFO("%20s: ", "File fek ");
		for(i = 0; i< TEE_FS_KM_FEK_SIZE; i++)
			printf("%2x ", fat_entry.fek[i]);
		printf("\r\n");
		RPMB_MSG_INFO("%20s: 0x%x \r\n", "File context size" , gp_head.ds_size);
		printf("\r\n");

	}

	if (strcmp(argv[1], "read") == 0){
		void *addr = 0;
		unsigned int size = 0x200000;	// 2M

		addr = (void *)simple_strtoul(argv[2], NULL, 16);
		ret = get_file_context_from_rpmb(argv[3], (char *)addr, size);
		if(ret < 0){
			RPMB_MSG_INFO("Get file %s from rpmb fail!\n", argv[3]);
			ret = CMD_RET_FAILURE;
		}else{
			ret = CMD_RET_SUCCESS;
			printf("\r\n");
			RPMB_MSG_INFO("Get file %s context from rpmb success!\n", argv[3]);
		}
	}
	
	return ret;
}

U_BOOT_CMD(
	tee_rpmb, 5, 0, do_tee_rpmb,
	"tee rpmb partition test interface",
	"\n"
	"tee_rpmb aloha  - get FW image version info from tee rpmb partition\n"
	"tee_rpmb list     - list file name store at tee rpmb partition\n"
	"tee_rpmb info file_name     -list file info store at tee rpmb partition\n"
	"tee_rpmb read ddr file_name     - read file_name from tee rpmb partition to ddr\n"
);
#endif
