#ifndef __FLASH_SPI_LIST_H__
#define __FLASH_SPI_LIST_H__

#define EMPTY_VALUE                0

// device id
#define	MX_25L12835D_128Mbit		0xc22018
#define MX_25L6406E_64Mbit			0xc22017

#define WINBOND_W32Q64_64Mbit		0xef4017
#define WINBOND_W25Q128FV_128Mbit	0xef4018
#define WINBOND_W25Q256FV_256Mbit	0xef4019
#define WINBOND_W25M512JV_512Mbit	0xef7119

#define GD25Q64C_64Mbit				0xc84017
#define GD25Q128C_128Mbit			0xc84018

#define F25L64QA_64Mbit				0x8c4117

#define EN25Q64_64Mbit				0x1c3017

typedef enum
{
    eMODE_READ_NORMAL = 0,
    eMODE_READ_NORMAL_FAST,
    eMODE_READ_DUAL_FAST,
    eMODE_READ_QUAD_FAST 
} read_mode;

typedef enum
{
    eMODE_WRITE_NORMAL = 0,
    eMODE_WRITE_QUAD_FAST 
} write_mode;

// QUAD MODE   RDSR(read status reg)   WRSR(write status reg)
#define CMD_QUAD_RDSR1            0x05
#define CMD_QUAD_RDSR2            0x35
#define CMD_QUAD_RDSR3            0x15
#define CMD_QUAD_WRSR1            0X01
#define CMD_QUAD_WRSR2            0X31
#define CMD_QUAD_WRSR3            0X11
#define BIT7                        0x80
#define BIT6                        0x40
#define BIT5                        0x20
#define BIT4                        0x10
#define BIT3                        0x08
#define BIT2                        0x04
#define BIT1                        0x02
#define BIT0                        0x01

// spinor_info_t->erase_capacity
#define ERASE_4K           0x01
#define ERASE_32K          0x02
#define ERASE_64K          0x04
#define ERASE_256K         0x08

#define ERASE_SZIE_64K  0x10000
#define ERASE_SZIE_32K  0x8000
#define ERASE_SZIE_4K   0x1000
// EARSE 
typedef struct
{
    unsigned sec_4k_en:1;
    unsigned sec_32k_en:1;
    unsigned sec_64k_en:1;
    unsigned sec_256k_en:1;
}erase_capacity_t;

typedef struct 
{
    unsigned int    id ;
    unsigned char   erase_capacity;  // ERASE_4K | ERASE_32K | ERASE_64K | ERASE_256K
    unsigned int    min_erase_size;
    unsigned char   page_program ;
    unsigned int    size;
    char   *string;
    read_mode       read_mode;
    write_mode      write_mode;
    
    /*quad mode*/
    unsigned char   cmd_quad_rdsr;
    unsigned char   cmd_quad_wrsr;
    unsigned char   cmd_quad_bit;

    /*4bytes address enable*/
    unsigned char	address_4bytes_enable;

    /*multi die flash*/
    unsigned char	multi_die_support;
    unsigned int	per_die_size;
} spinor_info_t;


static const spinor_info_t s_device[] = 
{
     // id                      256 64 32 4                    erase_size     page_program     size         string                       read_mode              write_mode
    {MX_25L12835D_128Mbit,       ERASE_4K|ERASE_64K ,          ERASE_SZIE_4K,    1,         0x1000000, "MX_25L12835D_128Mbit",      eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR1, CMD_QUAD_WRSR1, BIT6,         0,  0,  0},
    {MX_25L6406E_64Mbit,         ERASE_4K|ERASE_64K ,          ERASE_SZIE_4K,    1,         0x800000,  "MX_25L6406E_64Mbit",        eMODE_READ_DUAL_FAST,    eMODE_WRITE_NORMAL,  EMPTY_VALUE,    EMPTY_VALUE,    EMPTY_VALUE,  0,  0,  0},
    
    {WINBOND_W32Q64_64Mbit,      ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x800000,  "WINBOND_W32Q64_64Mbit",     eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR2, CMD_QUAD_WRSR1, BIT1,         0,  0,  0},
    {WINBOND_W25Q128FV_128Mbit,  ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x1000000, "WINBOND_W25Q128FV_128Mbit", eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR2, CMD_QUAD_WRSR2, BIT1,         0,  0,  0},

    {WINBOND_W25Q256FV_256Mbit,  ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x2000000, "WINBOND_W25Q256FV_256Mbit", eMODE_READ_QUAD_FAST,       eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR2, CMD_QUAD_WRSR2, BIT1,         1,  0,  0},
    {WINBOND_W25M512JV_512Mbit,  ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x4000000, "WINBOND_W25M512JV_512Mbit", eMODE_READ_DUAL_FAST,       eMODE_WRITE_NORMAL,  EMPTY_VALUE,    EMPTY_VALUE,    EMPTY_VALUE,  1,  1,  0x2000000},

    {GD25Q128C_128Mbit,          ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x1000000, "GD25Q128C_128Mbit",         eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR2, CMD_QUAD_WRSR2, BIT1,         0,  0,  0},
    {GD25Q64C_64Mbit,            ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x800000,  "GD25Q64C_64Mbit",           eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR2, CMD_QUAD_WRSR2, BIT1,         0,  0,  0},

    {F25L64QA_64Mbit,            ERASE_4K|ERASE_32K|ERASE_64K, ERASE_SZIE_4K,    1,         0x800000,  "F25L64QA_64Mbit",           eMODE_READ_QUAD_FAST,    eMODE_WRITE_NORMAL,  CMD_QUAD_RDSR1, CMD_QUAD_WRSR1, BIT6,         0,  0,  0},

} ; 

#define DEV_SIZE_S	(sizeof(s_device)/sizeof(spinor_info_t))

#endif /* __FLASH_SPI_LIST_H__ */
