/******************************************************************************
 * $Id: rtk_nand.c 337905 2010-10-18 01:32:27Z alexchang2131 $
 * drivers/mtd/nand/rtk_nand.c
 * Overview: Realtek NAND Flash Controller Driver
 * Copyright (c) 2008 Realtek Semiconductor Corp. All Rights Reserved.
 * Modification History:
 *    #000 2010-07-02 AlexChang   create file
 *
 *
 *******************************************************************************/

/************************************************************************
 *  Include files
 ************************************************************************/
#include <common.h>
#include <malloc.h>
#include <linux/mtd/rtk_nand.h>
#include <linux/mtd/mtd.h>
#include <linux/list.h>
#include <asm/io.h>
#include <linux/bitops.h>
#include <mtd/mtd-abi.h>
#include <linux/time.h>
#include <linux/string.h>

#include <rbus/nand_reg.h>
#include <asm/arch/sys_flash_layout.h>
#include <platform_lib/board/pcb_mgr.h>

/************************************************************************
 *  Definitions
 ************************************************************************/
#define BANNER  "Realtek NAND Flash Driver"
#define VERSION  "$Id: rtk_nand.c $"

#define CONFIG_REALTEK_PCBMGR
NF_DEBUG_SET_LEVEL(NF_DEBUG_LEVEL_INFO);


//Add for GPIO Setting
#define BIT(x) (1 << (x))
#define SETBITS(x,y) ((x) |= (y))
#define CLEARBITS(x,y) ((x) &= (~(y)))
#define SETBIT(x,y) SETBITS((x), (BIT((y))))
#define CLEARBIT(x,y) CLEARBITS((x), (BIT((y))))

#define NF_GPIO_OUT	(1)
#define NF_GPIO_IN	(0)
//#define NF_WP_ENABLE	(0)
//#define NF_WP_DISABLE	(1)


/************************************************************************
 *  Static variables
 ************************************************************************/
/*static for wp*/
static unsigned int g_regGPIODirBase = 0;
static unsigned int g_regGPIOOutBase = 0;

static int g_WP_en = 0;
static unsigned int g_NFWP_no = 0;
static int g_NFWP_pinType = 0;
#if  defined(CONFIG_REALTEK_PCBMGR)
static unsigned int g_NFWP_value_INV = 0;
static unsigned int g_NFWP_value_init = 0;
#endif
static unsigned int g_NFWP_value_en = 0;
static unsigned int g_NFWP_value_dis = 0;

static char is_NF_CP_Enable_read = 0;
static char is_NF_CP_Enable_write = 0;

/*static for cp*/
static unsigned int u32CP_mode = 0;
static unsigned int cpSel = 0;
static unsigned int flash_protection_root_key_index = 0;

static char u8regKey0[4] = {0,0,0,0};
static char u8regKey1[4] = {0,0,0,0};
static char u8regKey2[4] = {0,0,0,0};
static char u8regKey3[4] = {0,0,0,0};
static unsigned int *regKey0;
static unsigned int *regKey1;
static unsigned int *regKey2;
static unsigned int *regKey3;

/*static for params that access control*/
static unsigned int g_BootcodeSize = 0;
static unsigned int g_Factory_param_size = 0;

#ifdef CONFIG_MTD_RBT_TEST
int test_erase_fail_remap = 0;
int test_write_fail_remap = 0;
int test_write_fail_bbt = 0;
int test_read_warning_remap = 0;
int test_read_fail_remap  = 0;
#endif

/************************************************************************
 *  Public variables
 ************************************************************************/
unsigned char g_edo_mode_enable = 0;

/************************************************************************
 *  External variables
 ************************************************************************/

/************************************************************************
 *  External function prototypes
 ************************************************************************/
extern int pcb_mgr_get_enum_info_byname(char *enum_name, unsigned long long *value);
extern int nf_init_table(struct mtd_info *mtd);

/************************************************************************
 *  Static function prototypes
 ************************************************************************/
/* nand driver low-level functions */
static int rtk_read_oob(struct mtd_info *mtd, u16 chipnr, int page, int len, u_char *buf);
static int rtk_read_ecc_page(struct mtd_info *mtd, u16 chipnr, unsigned int page,
			u_char *data, u_char *oob_buf);
static int rtk_write_oob(struct mtd_info *mtd, u16 chipnr, int page, int len, const u_char *buf);
static int rtk_write_ecc_page(struct mtd_info *mtd, u16 chipnr, unsigned int page,
			const u_char *data, const u_char *oob_buf);
static int rtk_erase_block(struct mtd_info *mtd, u16 chipnr, int page);

/************************************************************************
 *  Function body
 ************************************************************************/
void setGPIOBit(unsigned int nGPIOBase, unsigned int nGPIOnum, unsigned int uBit)
{
	unsigned int whichReg = 0;
	unsigned int whichBit = 0;
	unsigned int temp;
	whichReg = nGPIOBase+4*(nGPIOnum/32);
	whichBit = nGPIOnum%32;

	temp = REG_READ_U32(whichReg);
	if(uBit)
		REG_WRITE_U32(whichReg,SETBIT(temp,whichBit));
	else
		REG_WRITE_U32(whichReg,CLEARBIT(temp,whichBit));
}

unsigned int getGPIOBit(unsigned int nGPIOBase, unsigned int nGPIOnum)
{
	unsigned int whichReg = 0;
	unsigned int whichBit = 0;
	unsigned int ret=0;
	whichReg = nGPIOBase+4*(nGPIOnum/32);
	whichBit = nGPIOnum%32;
	ret = REG_READ_U32(whichReg);
	ret>>=whichBit;
	ret&=(0x1);
	return ret;
}

unsigned int getGPIORegVal(unsigned int nGPIOBase, unsigned int nGPIOnum)
{
	unsigned int whichReg = 0;
	whichReg = nGPIOBase+4*(nGPIOnum/32);
	return REG_READ_U32(whichReg);
}

void disable_nand_wp(void)
{
	REG_WRITE_U32(0x18000854,(REG_READ_U32(0x18000854)|(0x0000f000)));
	REG_WRITE_U32(0x1801bd08,(REG_READ_U32(0x1801bd08)|(0x00040000))|0x1);
	REG_WRITE_U32(0x1801bd1c,(REG_READ_U32(0x1801bd1c)|(0x00040000))|0x1);
}


static void rtk_nand_select_chip(struct mtd_info *mtd, int chip)
{
	switch(chip) {
		case -1:
			REG_WRITE_U32(REG_PD,0xff);
			break;
		case 0:
		case 1:
		case 2:
		case 3:
			REG_WRITE_U32(REG_PD, ~(1 << chip));
			break;
		default:
			REG_WRITE_U32(REG_PD, ~(1 << 0));
	}
}

//timing setting
static void rtk_nand_set_timing(unsigned char t1, unsigned char t2, 
	unsigned char t3, unsigned char nf_div )
{
	unsigned int temp_reg = 0;
	//change NF freq divider (need to disabled NF clock when change divider) 
	REG_WRITE_U32(REG_CLOCK_ENABLE1,(REG_READ_U32(REG_CLOCK_ENABLE1)&(~0x00000004))|0x1); //merlin2 need set bit[0]-->write data,or fail
	temp_reg = REG_READ_U32(REG_NF_CKSEL);
	NAND_INFO("temp_reg0 = 0x%x\n",temp_reg);
	temp_reg &= 0xFFFFF8FF;
	temp_reg |= nf_div<<8;

	NAND_INFO("temp_reg1 = 0x%x\n",temp_reg);
	REG_WRITE_U32(REG_NF_CKSEL,temp_reg);
	NAND_INFO("REG_CLOCK_ENABLE1 0x%x = 0x%x REG_NF_CKSEL 0x%x = 0x%x\n",
		REG_CLOCK_ENABLE1,REG_READ_U32(REG_CLOCK_ENABLE1),REG_NF_CKSEL
		,REG_READ_U32(REG_NF_CKSEL));
	
	REG_WRITE_U32(REG_CLOCK_ENABLE1,(REG_READ_U32(REG_CLOCK_ENABLE1)|(0x00000004))|0x1); //merlin2 need set bit[0]-->write data,or fail
	// setup NF controller timing (lower speed to read flash ID)
	REG_WRITE_U32(REG_TIME_PARA3, t3);
	REG_WRITE_U32(REG_TIME_PARA2, t2);
	REG_WRITE_U32(REG_TIME_PARA1, t1);
	
	// setup T_WHR_ADL delay clock
	REG_WRITE_U32( REG_DELAY_CTL,NF_DELAY_CTL_T_WHR_ADL(0x09) );
}

//bootcode don't support float div/multi
static void rtk_nand_get_edo_mode(void)
{
	unsigned int  t2 = 0,t3 = 0;
	unsigned int nd_clk = 0;

	unsigned int sys_clk = 0;
	unsigned int div_num = 0;
	unsigned int  rate = 0;
	t2 = REG_READ_U32(REG_TIME_PARA2)&0xff;
	t3 = REG_READ_U32(REG_TIME_PARA3)&0xff;
	nd_clk = (REG_READ_U32(REG_NF_CKSEL)>>8) & 0x7;
	sys_clk = 295;	//295MHz
	div_num = nd_clk+3;
	//nd_clk_float = (float)295/((float)(nd_clk+3));
	
	rate =  sys_clk/(div_num*((t2+1) +(t3+1)));

	if(rate >= 33){
		NAND_INFO("[Clk = %dMHZ]Enable edo mode!\n", rate);
		g_edo_mode_enable = 1;
	}else{
		NAND_INFO("[Clk = %dMHZ]Disable edo mode!\n", rate);
		g_edo_mode_enable = 0;
	}
	return ;
}

static void rtk_nand_clock_select(char *PartNum, unsigned int value)
{
	unsigned int temp_reg = 0;
	//change NF freq divider (need to disabled NF clock when change divider) 
	REG_WRITE_U32(REG_CLOCK_ENABLE1,(REG_READ_U32(REG_CLOCK_ENABLE1)&(~0x00000004))|0x1); //merlin2 need set bit[0]-->write data,or fail
	temp_reg = REG_READ_U32(REG_NF_CKSEL);
	NAND_INFO("temp_reg0 = 0x%x\n",temp_reg);
	temp_reg &= 0xFFFFF8FF;
	temp_reg |= value<<8;

	NAND_INFO("temp_reg1 = 0x%x\n",temp_reg);
	REG_WRITE_U32(REG_NF_CKSEL,temp_reg);
	REG_WRITE_U32(REG_CLOCK_ENABLE1,(REG_READ_U32(REG_CLOCK_ENABLE1)|(0x00000004))|0x1); //merlin2 need set bit[0]-->write data,or fail
	NAND_INFO("%s is set to nf clock: 0x%x\n",PartNum, value);
}

//random mode utility func setting
static unsigned char nf_is_random_mode(void)
{
	return REG_READ_U32(REG_RND_EN) & 0x1;
}

static void nf_set_random_read(unsigned int pagesize,unsigned int spare_col)
{
	REG_WRITE_U32(REG_RND_CMD1, RND_CMD1_rnd_cmd1(0x5));
	REG_WRITE_U32(REG_RND_CMD2, RND_CMD2_rnd_cmd2(0xe0));
	REG_WRITE_U32(REG_RND_DATA_STR_COL_H, RND_DATA_STR_COL_H_data_col_msb(0));	// data start address MSB (always 0)
	REG_WRITE_U32(REG_RND_SPR_STR_COL_H, RND_SPR_STR_COL_H_spr_col_msb((spare_col+ pagesize) >> 8));	// spare start address MSB
	REG_WRITE_U32(REG_RND_SPR_STR_COL_L, REG_RND_SPR_STR_COL_L_spr_col_lsb((spare_col+ pagesize) & 0xff));	// spare start address LSB
}

static void nf_set_random_write(unsigned int pagesize,unsigned int spare_col)
{
	REG_WRITE_U32(REG_RND_CMD1, RND_CMD1_rnd_cmd1(0x85));
	REG_WRITE_U32(REG_RND_DATA_STR_COL_H, RND_DATA_STR_COL_H_data_col_msb(0));	// data start address MSB (always 0)
	REG_WRITE_U32(REG_RND_SPR_STR_COL_H, RND_SPR_STR_COL_H_spr_col_msb((spare_col+ pagesize) >> 8));	// spare start address MSB
	REG_WRITE_U32(REG_RND_SPR_STR_COL_L, REG_RND_SPR_STR_COL_L_spr_col_lsb((spare_col+ pagesize) & 0xff));	// spare start address LSB
}

void WAIT_DONE(unsigned int addr, unsigned int mask, unsigned int value)
{
	while ( (REG_READ_U32(addr) & mask) != value )
	{
		asm("nop");
	}
}

static void rtk_read_oob_from_SRAM(struct mtd_info *mtd, __u8 *r_oobbuf)
{
	struct nand_chip *this = (struct nand_chip *) mtd->priv;
	unsigned int reg_oob, reg_num;
	int i;
	unsigned int sram_base_addr;
	sram_base_addr = REG_NF_BASE_ADDR + 0x400;
	if ( mtd->ecctype == MTD_ECC_NONE ||(mtd->ecctype!=MTD_ECC_RTK_HW)){
		reg_num = sram_base_addr;
		reg_oob = REG_READ_U32(reg_num);
		r_oobbuf[0] = reg_oob & 0xff;
		r_oobbuf[1] = (reg_oob >> 8) & 0xff;
		r_oobbuf[2] = (reg_oob >> 16) & 0xff;
		r_oobbuf[3] = (reg_oob >> 24) & 0xff;

		reg_num = sram_base_addr+4;
		reg_oob = REG_READ_U32(reg_num);
		r_oobbuf[4] = reg_oob & 0xff;

		reg_num = sram_base_addr+16;
		reg_oob = REG_READ_U32(reg_num);
		r_oobbuf[5] = reg_oob & 0xff;
		r_oobbuf[6] = (reg_oob >> 8) & 0xff;
		r_oobbuf[7] = (reg_oob >> 16) & 0xff;
		r_oobbuf[8] = (reg_oob >> 24) & 0xff;

		reg_num = sram_base_addr+16*2;
		reg_oob = REG_READ_U32(reg_num);
		r_oobbuf[9] = reg_oob & 0xff;
		r_oobbuf[10] = (reg_oob >> 8) & 0xff;
		r_oobbuf[11] = (reg_oob >> 16) & 0xff;
		r_oobbuf[12] = (reg_oob >> 24) & 0xff;

		reg_num = sram_base_addr+16*3;
		reg_oob = REG_READ_U32(reg_num);
		r_oobbuf[13] = reg_oob & 0xff;
		r_oobbuf[14] = (reg_oob >> 8) & 0xff;
		r_oobbuf[15] = (reg_oob >> 16) & 0xff;
		r_oobbuf[16] = (reg_oob >> 24) & 0xff;
	}else{
		//just support 6/12/24 bit-ecc dump, Not support 40bit-ecc dump
		unsigned int increment = 0, len = 0, cnt = 0, sram_oob_size=0;
		unsigned int count_temp=0, j =0;
		//sanity check
		if(this->ecc_select == 0x28){
			NAND_ERR("Not support 40bit-ecc dump!!!\n");
			return;
		}

		// ecc-bit select
		switch (this->ecc_select) {
			case 0x18:			// 46B/1KB (4B for software)
				increment = 0x40;
				len = 46;
				cnt = mtd->oobblock/1024;
				break;
			case 0x0c:			// 26B/512B (6B for software)
				increment = 0x20;
				len = 26;
				cnt = mtd->oobblock/512;
				break;
			case 0x0:			// 16B/512B (6B for software)
			default:
				increment = 0x10;
				len = 16;
				cnt = mtd->oobblock/512;
				break;
		}

		if(is_296x_cpu())
			sram_oob_size = 256;
		else{
			sram_oob_size = 64;
		}

		for (i = 0,count_temp =0; (i < sram_oob_size) && (count_temp<cnt); i += increment,count_temp++) {
			for(j =0 ; j<len; j++){
				reg_num = sram_base_addr + j +i;
				r_oobbuf[i+j] = REG_READ_U8(reg_num);
			}
		}
	}
}

static void rtk_nand_read_id(struct mtd_info *mtd, u_char id[6])
{
	int id_chain;

#define XFER_MODE
	REG_WRITE_U32(REG_DATA_TL0,NF_DATA_TL0_length0(0x06) );
	REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_access_mode(0x01));

	REG_WRITE_U32(REG_READ_BY_PP,NF_READ_BY_PP_read_by_pp(0x0));
	REG_WRITE_U32(REG_PP_CTL0,NF_PP_CTL0_pp_enable(0x01));
	REG_WRITE_U32(REG_PP_CTL1,NF_PP_CTL1_pp_start_addr(0));

	REG_WRITE_U32(REG_ND_CMD,NF_ND_CMD_cmd(CMD_READ_ID));
#ifdef XFER_MODE
	REG_WRITE_U32(REG_ND_CTL,NF_ND_CTL_xfer(0x01));
	WAIT_DONE(REG_ND_CTL,0x80,0);
#endif
	REG_WRITE_U32(REG_ND_PA0,NF_ND_PA0_page_addr0(0));
	REG_WRITE_U32(REG_ND_PA2,NF_ND_PA2_addr_mode(0x07));

#ifdef XFER_MODE
	REG_WRITE_U32(REG_ND_CTL,NF_ND_CTL_xfer(1)|NF_ND_CTL_tran_mode(1));
	WAIT_DONE(REG_ND_CTL,0x80,0);
#else
    	REG_WRITE_U32( REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(1)| NF_AUTO_TRIG_auto_case(1));
	WAIT_DONE(REG_AUTO_TRIG,0x80,0);
#endif
	REG_WRITE_U32(REG_ND_CTL,NF_ND_CTL_xfer(1)|NF_ND_CTL_tran_mode(4));
	WAIT_DONE(REG_ND_CTL,0x80,0);

	REG_WRITE_U32(REG_PP_CTL0,NF_PP_CTL0_pp_reset(1));

	REG_WRITE_U32(REG_SRAM_CTL,NF_SRAM_CTL_map_sel(1)|NF_SRAM_CTL_access_en(1)|NF_SRAM_CTL_mem_region(0));

	id_chain = REG_READ_U32(REG_NF_BASE_ADDR+0x400);

	id[0] = id_chain & 0xff;
	id[1] = (id_chain >> 8) & 0xff;
	id[2] = (id_chain >> 16) & 0xff;
	id[3] = (id_chain >> 24) & 0xff;

	id_chain = REG_READ_U32(REG_NF_BASE_ADDR+0x404);
	
	id[4] = id_chain & 0xff;
	id[5] = (id_chain >> 8) & 0xff;

	REG_WRITE_U32(REG_SRAM_CTL,0x0);	//# no omitted

}

static void reverse_to_Tags(u_char *r_oobbuf, int eccBits)
{
	int k,j;
	if((eccBits==0)||(eccBits==0xc)){
		//6bit-ecc & 12bit-ecc
		for ( k=0; k<4; k++ )
			r_oobbuf[5+k] = r_oobbuf[8+k];

		memcpy(&r_oobbuf[9],&r_oobbuf[16],4);//add by alexchang for improve nand read 0225-2010
		memcpy(&r_oobbuf[13],&r_oobbuf[24],4);//add by alexchang for improve nand read 0225-2010
	}else if((eccBits==0x18)||(eccBits==0x28)){
		//24bit-ecc & 40bit-ecc
		for(k=0,j=1;k<28;k+=4,j++)
			memcpy(&r_oobbuf[1+k],&r_oobbuf[8*j],4);
	}
}

static int rtk_erase_block(struct mtd_info *mtd, u16 chipnr, int page)
{
	struct nand_chip *this = (struct nand_chip *) mtd->priv;
#ifdef CONFIG_MTD_RBT_TEST
	if(test_erase_fail_remap != 0){
		NAND_ERR("page %d block %d enter erase remap test mode!!\n", 
			page,page/this->ppb);
		test_erase_fail_remap = 0;
		return -1;
	}
#endif
	if ( page & (this->ppb-1) ){
		NAND_ERR("page %d is not block alignment !!\n", page);
		return -1;
	}

	if(g_edo_mode_enable)
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(1));
	else
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));

	if(g_WP_en){
		switch(g_NFWP_pinType)
		{
			case PCB_PIN_TYPE_GPIO:
			default:
				setGPIOBit(g_regGPIODirBase,g_NFWP_no,NF_GPIO_OUT);//Set Driection
				setGPIOBit(g_regGPIOOutBase,g_NFWP_no,g_NFWP_value_dis);//Set Value
				break;
		}
		REG_WRITE_U32( REG_CMD3,NF_CMD3_cmd3(CMD_BLK_ERASE_C3));
		REG_WRITE_U32( REG_POLL_FSTS,NF_POLL_FSTS_bit_sel(0x07)|NF_POLL_FSTS_trig_poll(1));
		WAIT_DONE(REG_POLL_FSTS,0x1,0);
	}

	REG_WRITE_U32( REG_ND_CMD,NF_ND_CMD_cmd(CMD_BLK_ERASE_C1) );
	REG_WRITE_U32( REG_CMD2,NF_CMD2_cmd2(CMD_BLK_ERASE_C2) );
	REG_WRITE_U32( REG_CMD3,NF_CMD3_cmd3(CMD_BLK_ERASE_C3) );

	REG_WRITE_U32( REG_ND_PA0,NF_ND_PA0_page_addr0(page) );
	REG_WRITE_U32( REG_ND_PA1,NF_ND_PA1_page_addr1(page>>8) );
	REG_WRITE_U32( REG_ND_PA2,NF_ND_PA2_addr_mode(0x04)|NF_ND_PA2_page_addr2(page>>16) );
	REG_WRITE_U32( REG_ND_PA3,NF_ND_PA3_page_addr3((page>> 21)&0x7) );
	REG_WRITE_U32(REG_ND_CA0,NF_ND_CA0_col_addr0(0));
	REG_WRITE_U32(REG_ND_CA1,NF_ND_CA1_col_addr1(0));

	REG_WRITE_U32( REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(1)|NF_AUTO_TRIG_auto_case(2) );
	WAIT_DONE(REG_AUTO_TRIG,0x80,0);

	REG_WRITE_U32(REG_POLL_FSTS,NF_POLL_FSTS_bit_sel(6)|NF_POLL_FSTS_trig_poll(1) );
	WAIT_DONE(REG_POLL_FSTS,0x01,0x0);
	WAIT_DONE(REG_ND_CTL,0x40,0x40);//add by alexchang 0416-2010

	if(g_WP_en){
		switch(g_NFWP_pinType)
		{
			case PCB_PIN_TYPE_GPIO:
			default:
				setGPIOBit(g_regGPIODirBase,g_NFWP_no,NF_GPIO_OUT);//Set Driection
				setGPIOBit(g_regGPIOOutBase,g_NFWP_no,g_NFWP_value_en);//Set Value
				break;
		}
	}

	if ( REG_READ_U32(REG_ND_DAT) & 0x01 ){
		NAND_ERR("erasure is not completed at block %d\n", page/this->ppb);
		return -1;
	}

	return 0;
}

unsigned char *malloc_aligned(size_t size,int alignment)
{
	unsigned char *ptr = kmalloc(size + alignment-1,GFP_KERNEL);

	if (!ptr)
		return NULL;

	ptr = (unsigned char *)(((unsigned int)(ptr) + alignment-1) & ~alignment);

	return ptr;
}

static int rtk_read_oob (struct mtd_info *mtd, u16 chipnr, int page, int len,
	u_char *oob_buf)
{
	struct nand_chip *this = (struct nand_chip *) mtd->priv;
	int rc = 0;
	int dram_sa, dma_len, spare_dram_sa;
	int blank_all_one=0;
	int page_len;
	uint8_t auto_trigger_mode = 2;
	uint8_t addr_mode = 1;
	this->ecc_err_flag = 0;

	if ( len>this->oob_size || !oob_buf ){
		NAND_ERR("len>oob_size OR oob_buf is NULL\n");
		return -1;
	}
	if ( chipnr == 0 && page >= 0 && page < g_BootcodeSize/this->page_size )
		mtd->isCPdisable_R = 1;

	if (nf_is_random_mode()){
		nf_set_random_read(mtd->oobblock, 0);
	}

	if(g_edo_mode_enable)
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(1));
	else
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));

	REG_WRITE_U32(REG_BLANK_CHK,NF_BLANK_CHK_blank_ena(1));
	REG_WRITE_U32(REG_DATA_TL0,NF_DATA_TL0_length0(0)); //REG:0x100
	if((this->ecc_select == 0x18)||(this->ecc_select == 0x28))
	{
		page_len = this->page_size >> 10;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_access_mode(1)|NF_DATA_TL1_length1(4));
	}else{
		page_len = this->page_size >> 9;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_access_mode(1)|NF_DATA_TL1_length1(2));
	}
	REG_WRITE_U32(REG_PAGE_LEN,NF_PAGE_LEN_page_len(page_len));

	REG_WRITE_U32(REG_READ_BY_PP,NF_READ_BY_PP_read_by_pp(1));//add by alexchang 0208-2010
	REG_WRITE_U32(REG_PP_CTL1,NF_PP_CTL1_pp_start_addr(0));//add by alexchang 0208-2010
	REG_WRITE_U32(REG_PP_CTL0,0);//add by alexchang 0208-2010

	if ( this->ecc_select == 0xc)	//setting for 12bits ecc.
 		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(1));
	else if( this->ecc_select == 0x18) //setting for 24bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(1)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else if(this->ecc_select == 0x28)	//setting for 40bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(1)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));

	REG_WRITE_U32(REG_ND_CMD,NF_ND_CMD_cmd(CMD_PG_READ_C1));
	REG_WRITE_U32(REG_CMD2,NF_CMD2_cmd2(CMD_PG_READ_C2));
	REG_WRITE_U32(REG_CMD3,NF_CMD3_cmd3(CMD_PG_READ_C3));

	REG_WRITE_U32( REG_ND_PA0,NF_ND_PA0_page_addr0(page) );
	REG_WRITE_U32( REG_ND_PA1,NF_ND_PA1_page_addr1(page>>8) );
	REG_WRITE_U32( REG_ND_PA2,NF_ND_PA2_addr_mode(addr_mode)|NF_ND_PA2_page_addr2(page>>16) );
	REG_WRITE_U32( REG_ND_PA3,NF_ND_PA3_page_addr3((page>>21)&0x7) );
	REG_WRITE_U32(REG_ND_CA0,NF_ND_CA0_col_addr0(0));
	REG_WRITE_U32(REG_ND_CA1,NF_ND_CA1_col_addr1(0));

	//Set ECC
	REG_WRITE_U32(REG_ECC_STOP,NF_ECC_STOP_ecc_n_stop(1));//add by alexchang0223-2010

	//Set DMA
	dram_sa = ( (uint32_t) this->g_databuf >> 3);
	REG_WRITE_U32(REG_DMA_CTL1,NF_DMA_CTL1_dram_sa(dram_sa));

	dma_len = this->page_size >> 9;
	REG_WRITE_U32(REG_DMA_CTL2,NF_DMA_CTL2_dma_len(dma_len));

	spare_dram_sa = ( (uint32_t)oob_buf >> 3);		//oob
	REG_WRITE_U32( REG_SPR_DDR_CTL,NF_SPR_DDR_CTL_cr_nf_hw_pinmux_ena(1)|NF_SPR_DDR_CTL_spare_ddr_ena(1)|NF_SPR_DDR_CTL_per_2k_spr_ena(1)|NF_SPR_DDR_CTL_spare_dram_sa(spare_dram_sa));

	if(is_NF_CP_Enable_read&&(!mtd->isCPdisable_R))
	{
		if(cpSel == 0){
			//register mode
			REG_WRITE_U32(REG_CP_NF_KEY,*regKey0);
			REG_WRITE_U32((REG_CP_NF_KEY+0x4),*regKey1);
			REG_WRITE_U32((REG_CP_NF_KEY+0x8),*regKey2);
			REG_WRITE_U32((REG_CP_NF_KEY+0xc),*regKey3);
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode|cpSel);
		}else if(cpSel == 1){
			//cw mode
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode |(flash_protection_root_key_index << 2) |cpSel);
		}else{
			//otp mode
			NAND_INFO("[Warning]Not set otp mode!\n");
		}
		REG_WRITE_U32(REG_CP_LEN,NF_CP_LEN_cp_length(this->page_size));
		RTK_FLUSH_CACHE((unsigned long) this->g_databuf, this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);

		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0x1)|NF_DMA_CTL3_cp_first(0x1)|NF_DMA_CTL3_ddr_wr(0x1)|NF_DMA_CTL3_dma_xfer(0x1));
	}
	else
	{
		RTK_FLUSH_CACHE((unsigned long) this->g_databuf, this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_ddr_wr(1)|NF_DMA_CTL3_dma_xfer(1));
	}

	REG_WRITE_U32( REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(0)| NF_AUTO_TRIG_auto_case(auto_trigger_mode));

	WAIT_DONE(REG_AUTO_TRIG,0x80,0);
	WAIT_DONE(REG_DMA_CTL3,0x01,0);

	WAIT_DONE(REG_ND_CTL,0x40,0x40);//add by alexchang 0416-2010	

	if(is_NF_CP_Enable_read&&(!mtd->isCPdisable_R))//Clear register
	{
		REG_WRITE_U32(REG_CP_NF_SET,0);
	}

	if(oob_buf)
	{
		if(this->ecc_select==0x28 ||this->ecc_select==0x18||this->ecc_select==0xc||is_296x_cpu())
			reverse_to_Tags(oob_buf,this->ecc_select);
		else
		{
			REG_WRITE_U32(REG_READ_BY_PP,0x00);
			REG_WRITE_U32(REG_SRAM_CTL, 0x30 | 0x04);
			rtk_read_oob_from_SRAM(mtd, oob_buf);
			REG_WRITE_U32(REG_SRAM_CTL, 0x00);
			REG_WRITE_U32(REG_READ_BY_PP,0x80);
			reverse_to_Tags(oob_buf,this->ecc_select);
		}
	}

	blank_all_one = (REG_READ_U32(REG_BLANK_CHK)>>1) & 0x01;

	if(blank_all_one == 1){
		// return OK if all data bit is 1 (page is not written yet)
		return 0;
	}

	if (REG_READ_U32(REG_ND_ECC) & 0x0C){
		if (REG_READ_U32(REG_ND_ECC) & 0x08){
			NAND_ERR("blk %d, page %d fail\n",
				page/this->ppb,page%this->ppb);
			return -1;
		}
		if(REG_READ_U32(REG_ND_ECC) & 0x04){
			if((REG_READ_U32(REG_MAX_ECC_NUM) & 0x3F) > this->ecc_safe_value){
				NAND_INFO("blk %d, page %d ecc error warning,ecc_num = 0x%x\n",
					page/this->ppb,page%this->ppb,
					REG_READ_U32(REG_RSECC_NUM));
				this->ecc_err_flag = 1;
			}
		}		
	}

	mtd->isCPdisable_R = 0;

	return rc;
}

static int rtk_read_ecc_page (struct mtd_info *mtd, u16 chipnr, unsigned int page,
	u_char *data_buf, u_char *oob_buf)
{
	struct nand_chip *this = NULL;
	int rc = 0;
	int dram_sa, dma_len, spare_dram_sa;
	int blank_all_one = 0;
	int page_len;
	uint8_t	auto_trigger_mode = 2;
	uint8_t	addr_mode = 1;
	this = (struct nand_chip *) mtd->priv;
	u_char *read_data_buf = NULL;
	this->ecc_err_flag = 0;

#ifdef CONFIG_MTD_RBT_TEST
	if(test_read_fail_remap == 1){
		test_read_fail_remap = 0;
		NAND_INFO("enter page %d block %d read fail ecc error case! \n",
			page,page/this->ppb);
		return -1;
	}
#endif

        if(((uint32_t)data_buf&0x7)!=0){
			NAND_INFO("data_buf 0x%x must 8 byte alignmemt read buf 0x%x!!\n",
			(uint32_t)data_buf,(uint32_t)this->g_pagebuf);
			memset(this->g_pagebuf,0xFF,this->page_size);
			read_data_buf = this->g_pagebuf;
        }else{
        	read_data_buf = data_buf;
        }

	if (nf_is_random_mode()){
		nf_set_random_read(mtd->oobblock, 0);
	}

	if(g_edo_mode_enable)
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(1));
	else
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));

	REG_WRITE_U32(REG_BLANK_CHK,NF_BLANK_CHK_blank_ena(1) );
	REG_WRITE_U32(REG_DATA_TL0,NF_DATA_TL0_length0(0));

	if((this->ecc_select == 0x18)||(this->ecc_select == 0x28)){
		page_len = this->page_size >> 10;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_access_mode(1)|NF_DATA_TL1_length1(4));
	}else{
		page_len = this->page_size >> 9;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_access_mode(1)|NF_DATA_TL1_length1(2));
	}
	REG_WRITE_U32(REG_PAGE_LEN,NF_PAGE_LEN_page_len(page_len));

	REG_WRITE_U32(REG_READ_BY_PP,NF_READ_BY_PP_read_by_pp(1));
	REG_WRITE_U32(REG_PP_CTL1,NF_PP_CTL1_pp_start_addr(0));
	REG_WRITE_U32(REG_PP_CTL0,0);

 	if ( this->ecc_select == 0xc)	//setting for 12bits ecc.
 		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(1));
	else if( this->ecc_select == 0x18) //setting for 24bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(1)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else if(this->ecc_select == 0x28)	//setting for 40bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(1)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));

	REG_WRITE_U32(REG_ND_CMD,NF_ND_CMD_cmd(CMD_PG_READ_C1));
	REG_WRITE_U32(REG_CMD2,NF_CMD2_cmd2(CMD_PG_READ_C2));
	REG_WRITE_U32(REG_CMD3,NF_CMD3_cmd3(CMD_PG_READ_C3));

	REG_WRITE_U32( REG_ND_PA0,NF_ND_PA0_page_addr0(page));
	REG_WRITE_U32( REG_ND_PA1,NF_ND_PA1_page_addr1(page>>8));
	REG_WRITE_U32( REG_ND_PA2,NF_ND_PA2_addr_mode(addr_mode)|NF_ND_PA2_page_addr2(page>>16));
	REG_WRITE_U32( REG_ND_PA3,NF_ND_PA3_page_addr3((page>>21)&0x7));
	REG_WRITE_U32(REG_ND_CA0,NF_ND_CA0_col_addr0(0));
	REG_WRITE_U32(REG_ND_CA1,NF_ND_CA1_col_addr1(0));

	REG_WRITE_U32( REG_ECC_STOP,NF_ECC_STOP_ecc_n_stop(0x01));

	dma_len = this->page_size >> 9;
	REG_WRITE_U32(REG_DMA_CTL2,NF_DMA_CTL2_dma_len(dma_len));

	dram_sa = ( (uint32_t)read_data_buf >> 3);
	REG_WRITE_U32(REG_DMA_CTL1,NF_DMA_CTL1_dram_sa(dram_sa));

	if (oob_buf)
		spare_dram_sa = ( (uint32_t)oob_buf >> 3);
	else
		spare_dram_sa = ( (uint32_t)this->g_oobbuf >> 3);

	//patch
	RTK_FLUSH_CACHE((unsigned long) read_data_buf, this->page_size);
	if ( oob_buf )
		RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
	else
		RTK_FLUSH_CACHE((unsigned long) this->g_oobbuf, this->oob_size);

	REG_WRITE_U32( REG_SPR_DDR_CTL,NF_SPR_DDR_CTL_cr_nf_hw_pinmux_ena(1)|NF_SPR_DDR_CTL_spare_ddr_ena(1)|NF_SPR_DDR_CTL_per_2k_spr_ena(1)|NF_SPR_DDR_CTL_spare_dram_sa(spare_dram_sa));

	if(is_NF_CP_Enable_read&&(!mtd->isCPdisable_R)){
		if(cpSel == 0){
			//register mode
			REG_WRITE_U32(REG_CP_NF_KEY,*regKey0);
			REG_WRITE_U32((REG_CP_NF_KEY+0x4),*regKey1);
			REG_WRITE_U32((REG_CP_NF_KEY+0x8),*regKey2);
			REG_WRITE_U32((REG_CP_NF_KEY+0xc),*regKey3);
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode|cpSel);
		}else if(cpSel == 1){
			//cw mode
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode |(flash_protection_root_key_index << 2) |cpSel);
		}else{
			//otp mode
			NAND_INFO("[Warning]Not set otp mode!\n");
		}
		REG_WRITE_U32(REG_CP_LEN,NF_CP_LEN_cp_length(this->page_size));
		RTK_FLUSH_CACHE((unsigned long) read_data_buf, this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
		else
			RTK_FLUSH_CACHE((unsigned long) this->g_oobbuf, this->oob_size);
		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0x1)|NF_DMA_CTL3_cp_first(0x1)|NF_DMA_CTL3_ddr_wr(0x1)|NF_DMA_CTL3_dma_xfer(0x1));
	}else{
		RTK_FLUSH_CACHE((unsigned long) read_data_buf,this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
		else
			RTK_FLUSH_CACHE((unsigned long) this->g_oobbuf, this->oob_size);
		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0)|NF_DMA_CTL3_cp_first(0)|NF_DMA_CTL3_ddr_wr(0x1)|NF_DMA_CTL3_dma_xfer(0x1));
	}

	REG_WRITE_U32( REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(0)| NF_AUTO_TRIG_auto_case(auto_trigger_mode));

	WAIT_DONE(REG_AUTO_TRIG,0x80,0);
	WAIT_DONE(REG_DMA_CTL3,0x01,0);

	WAIT_DONE(REG_ND_CTL,0x40,0x40);//add by alexchang 0416-2010	

	if(is_NF_CP_Enable_read&&(!mtd->isCPdisable_R)){//Clear register
		REG_WRITE_U32(REG_CP_NF_SET,0);
	}

	if(oob_buf)
	{
		if(this->ecc_select==0x28||this->ecc_select==0x18||this->ecc_select==0x0c||is_296x_cpu()){
			if(mtd->ecctype!=MTD_ECC_RTK_HW)
				reverse_to_Tags(oob_buf,this->ecc_select);
			else{
				REG_WRITE_U32(REG_READ_BY_PP,0x00);
				REG_WRITE_U32(REG_SRAM_CTL, 0x30 | 0x04);
				rtk_read_oob_from_SRAM(mtd, oob_buf);
				REG_WRITE_U32(REG_SRAM_CTL, 0x00);
				REG_WRITE_U32(REG_READ_BY_PP,0x80);
			}
		}else{
			REG_WRITE_U32(REG_READ_BY_PP,0x00);
			REG_WRITE_U32(REG_SRAM_CTL, 0x30 | 0x04);
			rtk_read_oob_from_SRAM(mtd, oob_buf);
			REG_WRITE_U32(REG_SRAM_CTL, 0x00);
			REG_WRITE_U32(REG_READ_BY_PP,0x80);
			reverse_to_Tags(oob_buf,this->ecc_select);
		}
	}

	blank_all_one = (REG_READ_U32(REG_BLANK_CHK)>>1) & 0x01;

	mtd->nEccMaxValue = REG_MAX_ECC_NUM_max_ecc_num(REG_READ_U32(REG_MAX_ECC_NUM));
	if(((uint32_t)data_buf&0x7)!=0){
		memcpy(data_buf,read_data_buf,this->page_size);
	}
	if(blank_all_one == 1){
		// return OK if all data bit is 1 (page is not written yet)
		return 0;
	}

	if (REG_READ_U32(REG_ND_ECC) & 0x0C){
		if (REG_READ_U32(REG_ND_ECC) & 0x08){
			NAND_ERR("blk %d, page %d fail\n",page/this->ppb,page%this->ppb);
			return -1;
		}
		if(REG_READ_U32(REG_ND_ECC) & 0x04){
			if((REG_READ_U32(REG_MAX_ECC_NUM) & 0x3F) > this->ecc_safe_value){
				NAND_ERR("blk %d, page %d ecc error warning,ecc_num = 0x%x\n",
				page/this->ppb,page%this->ppb,REG_READ_U32(REG_RSECC_NUM));
				this->ecc_err_flag = 1;
			}
		}
	}
#ifdef CONFIG_MTD_RBT_TEST
	if(test_read_warning_remap != 0){
		NAND_INFO("enter test_read_warning_remap page %d block %d test mode!!\n",
			page,page/this->ppb);
		this->ecc_err_flag = 1;
		test_read_warning_remap = 0;
	}
#endif

	mtd->isCPdisable_R = 0;
	return rc;
}

static int rtk_write_oob(struct mtd_info *mtd, u16 chipnr, int page, int len, 
	const u_char *oob_buf)
{
	struct nand_chip *this = (struct nand_chip *) mtd->priv;
	int rc = 0;
	uint8_t	auto_trigger_mode = 1;
	uint8_t	addr_mode = 1;
	unsigned int page_len, dram_sa, dma_len, spare_dram_sa;

	memset(this->g_databuf, 0xff, this->page_size);
	if ( len>this->oob_size || !oob_buf ){
		NAND_ERR("len>oob_size OR oob_buf is NULL\n");
		return -1;
	}

	//----------------set random mode --------------------
	if (nf_is_random_mode()){
		nf_set_random_write(mtd->oobblock,0);
	}

	if(g_edo_mode_enable)
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(1));
	else
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));

	if(g_WP_en){
		switch(g_NFWP_pinType)
		{
			case PCB_PIN_TYPE_GPIO:
			default:
				setGPIOBit(g_regGPIODirBase,g_NFWP_no,NF_GPIO_OUT);//Set Driection
				setGPIOBit(g_regGPIOOutBase,g_NFWP_no,g_NFWP_value_dis);//Set Value
				break;
		}
		REG_WRITE_U32( REG_CMD3,NF_CMD3_cmd3(CMD_BLK_ERASE_C3));
		REG_WRITE_U32( REG_POLL_FSTS,NF_POLL_FSTS_bit_sel(0x07)|NF_POLL_FSTS_trig_poll(1));
		WAIT_DONE(REG_POLL_FSTS,0x1,0);
	}

	REG_WRITE_U32(REG_SRAM_CTL,0x00);//add by alexchang 0208-2010
	REG_WRITE_U32(REG_DATA_TL0,NF_DATA_TL0_length0(0));
 	if((this->ecc_select == 0x18)||(this->ecc_select == 0x28))
	{
		page_len = this->page_size >> 10;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_length1(4));
	}else{
		page_len = this->page_size >> 9;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_length1(2));
	}

	REG_WRITE_U32(REG_READ_BY_PP,NF_READ_BY_PP_read_by_pp(0));//add by alexchang 0208-2010
	REG_WRITE_U32(REG_PP_CTL1,NF_PP_CTL1_pp_start_addr(0));//add by alexchang 0208-2010
	REG_WRITE_U32(REG_PP_CTL0,0);//add by alexchang 0208-2010

	REG_WRITE_U32( REG_PAGE_LEN,NF_PAGE_LEN_page_len(page_len));

	if ( this->ecc_select == 0xc)	//setting for 12bits ecc.
 		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(1));
	else if( this->ecc_select == 0x18) //setting for 24bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(1)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else if(this->ecc_select == 0x28)	//setting for 40bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(1)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));

 	REG_WRITE_U32(REG_ND_CMD,NF_ND_CMD_cmd(CMD_PG_WRITE_C1));
	REG_WRITE_U32(REG_CMD2,NF_CMD2_cmd2(CMD_PG_WRITE_C2));
	REG_WRITE_U32(REG_CMD3,NF_CMD3_cmd3(CMD_PG_WRITE_C3));

	REG_WRITE_U32( REG_ND_PA0,NF_ND_PA0_page_addr0(page));
	REG_WRITE_U32( REG_ND_PA1,NF_ND_PA1_page_addr1(page>>8));
	REG_WRITE_U32( REG_ND_PA2,NF_ND_PA2_addr_mode(addr_mode)|NF_ND_PA2_page_addr2(page>>16));
	REG_WRITE_U32( REG_ND_PA3,NF_ND_PA3_page_addr3((page>>21)&0x7));
	REG_WRITE_U32(REG_ND_CA0,NF_ND_CA0_col_addr0(0));
	REG_WRITE_U32(REG_ND_CA1,NF_ND_CA1_col_addr1(0));

	dma_len = this->page_size >> 9;
	REG_WRITE_U32(REG_DMA_CTL2,NF_DMA_CTL2_dma_len(dma_len));

	dram_sa = ( (uint32_t)this->g_databuf >> 3);
	REG_WRITE_U32(REG_DMA_CTL1,NF_DMA_CTL1_dram_sa(dram_sa));

	spare_dram_sa = ( (uint32_t)oob_buf >> 3);
	REG_WRITE_U32( REG_SPR_DDR_CTL,NF_SPR_DDR_CTL_cr_nf_hw_pinmux_ena(1)|NF_SPR_DDR_CTL_spare_ddr_ena(1)|NF_SPR_DDR_CTL_per_2k_spr_ena(1)|NF_SPR_DDR_CTL_spare_dram_sa(spare_dram_sa));

	if(is_NF_CP_Enable_write&&(!mtd->isCPdisable_W))
	{
		if(cpSel == 0){
			//register mode
			REG_WRITE_U32(REG_CP_NF_KEY,*regKey0);
			REG_WRITE_U32((REG_CP_NF_KEY+0x4),*regKey1);
			REG_WRITE_U32((REG_CP_NF_KEY+0x8),*regKey2);
			REG_WRITE_U32((REG_CP_NF_KEY+0xc),*regKey3);
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode|cpSel);
		}else if(cpSel == 1){
			//cw mode
			REG_WRITE_U32(REG_CP_NF_SET,u32CP_mode |(flash_protection_root_key_index << 2) |cpSel);
		}else{
			//otp mode
			NAND_INFO("[Warning]Not set otp mode!\n");
		}
		REG_WRITE_U32(REG_CP_LEN,NF_CP_LEN_cp_length(this->page_size));

		RTK_FLUSH_CACHE((unsigned long) this->g_databuf, this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0x1)|NF_DMA_CTL3_cp_first(0x1)|NF_DMA_CTL3_ddr_wr(0)|NF_DMA_CTL3_dma_xfer(0x1));
	}else{
		RTK_FLUSH_CACHE((unsigned long) this->g_databuf, this->page_size);
		if ( oob_buf )
			RTK_FLUSH_CACHE((unsigned long) oob_buf, this->oob_size);
		REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0)|NF_DMA_CTL3_cp_first(0)|NF_DMA_CTL3_ddr_wr(0)|NF_DMA_CTL3_dma_xfer(0x1));
	}

	REG_WRITE_U32( REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(0)| NF_AUTO_TRIG_auto_case(auto_trigger_mode));

	WAIT_DONE(REG_AUTO_TRIG,0x80,0);
	WAIT_DONE(REG_DMA_CTL3,0x01,0);
	REG_WRITE_U32(REG_POLL_FSTS,NF_POLL_FSTS_bit_sel(6)|NF_POLL_FSTS_trig_poll(1) );
	WAIT_DONE(REG_POLL_FSTS,0x01,0x0);
	WAIT_DONE(REG_ND_CTL,0x40,0x40);//add by alexchang 0416-2010

	if(g_WP_en){
		switch(g_NFWP_pinType)
		{
			case PCB_PIN_TYPE_GPIO:
			default:
				setGPIOBit(g_regGPIODirBase,g_NFWP_no,NF_GPIO_OUT);//Set Driection
				setGPIOBit(g_regGPIOOutBase,g_NFWP_no,g_NFWP_value_en);//Set Value
				break;
		}
	}

	if(is_NF_CP_Enable_write&&(!mtd->isCPdisable_W)){//Clear register
		REG_WRITE_U32(REG_CP_NF_SET,0);
	}

	if ( REG_READ_U32(REG_ND_DAT) & 0x01 ){
		NAND_ERR("write oob is not completed at page %d\n", page);
		return -1;
	}

	mtd->isCPdisable_W = 0;
	return rc;
}

//----------------------------------------------------------------------------
//char bbbbb[4096];
unsigned char nf_oob_buf_temp[64*8];		//allocate
static int rtk_write_ecc_page (struct mtd_info *mtd, u16 chipnr, unsigned int page,
			const u_char *data_buf, const  u_char *oob_buf)
{
	struct nand_chip *this = (struct nand_chip *) mtd->priv;
	int rc = 0;
	uint8_t	auto_trigger_mode = 1;
	uint8_t	addr_mode = 1;
	unsigned int page_len, dram_sa, dma_len, spare_dram_sa;
	unsigned char *write_page_buf = NULL;
	unsigned char *nf_oob_buf = (unsigned char *)((unsigned long)nf_oob_buf_temp-(unsigned long)nf_oob_buf_temp%8+8);

        if(((uint32_t)data_buf&0x7)!=0){
			NAND_INFO("data_buf 0x%x must 8 byte alignmemt!! write_page_buf 0x%x!\n",
				(unsigned int)data_buf,(unsigned int)this->g_pagebuf);
			write_page_buf = this->g_pagebuf;
			memcpy(write_page_buf,data_buf,this->page_size);
        }else{
        	write_page_buf = (u_char *)data_buf;
        }
	if(((uint32_t)nf_oob_buf&0x7)!=0){
                NAND_ERR("oob_buf must 8 byte alignmemt!![nf_oob_buf = %x]\n",(unsigned int)nf_oob_buf);
                BUG();
        }
#ifdef CONFIG_MTD_RBT_TEST
	if(((test_write_fail_remap != 0)&&
		(page > (this->bi_table_block_start+this->bi_table_block_num)*this->ppb))||
		(test_write_fail_bbt != 0)){
		NAND_INFO("page %d block %d enter write remap test mode remap %d bbt %d!!\n",
			page,page/this->ppb,test_write_fail_remap,test_write_fail_bbt);
		if(test_write_fail_remap > 0){
			test_write_fail_remap--;
		}
		if(test_write_fail_bbt){
			test_write_fail_bbt--;
		}
		return -1;
	}
#endif
	memset(nf_oob_buf, 0xff, this->oob_size);

	//todo
	//setting the area that cp

	if (nf_is_random_mode()){
		nf_set_random_write(mtd->oobblock,0);
	}

	if(g_edo_mode_enable)
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(1));
	else
		REG_WRITE_U32(REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));
	if(oob_buf)
		memcpy(nf_oob_buf,oob_buf,this->oob_size);

	REG_WRITE_U32(REG_SRAM_CTL,0x00);//add by alexchang 0208-2010
	REG_WRITE_U32(REG_DATA_TL0,NF_DATA_TL0_length0(0));
 	if((this->ecc_select == 0x18)||(this->ecc_select == 0x28)){
		page_len = this->page_size >> 10;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_length1(4));
	}else{
		page_len = this->page_size >> 9;
		REG_WRITE_U32(REG_DATA_TL1,NF_DATA_TL1_length1(2));
	}

	REG_WRITE_U32( REG_PAGE_LEN,NF_PAGE_LEN_page_len(page_len));
	REG_WRITE_U32(REG_READ_BY_PP,NF_READ_BY_PP_read_by_pp(0));//add by alexchang 0208-2010
	REG_WRITE_U32(REG_PP_CTL0,0);
	REG_WRITE_U32(REG_PP_CTL1,NF_PP_CTL1_pp_start_addr(0));//add by alexchang 0208-2010

	REG_WRITE_U32( REG_ECC_STOP,NF_ECC_STOP_ecc_n_stop(0x01));
	if ( this->ecc_select == 0xc)	//setting for 12bits ecc.
 		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(1));
	else if( this->ecc_select == 0x18) //setting for 24bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(1)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else if(this->ecc_select == 0x28)	//setting for 40bits ecc.
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(1)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));
	else
		REG_WRITE_U32(REG_ECC_SEL,NF_ECC_SEL_ecc_bch_40b_ena(0)|NF_ECC_SEL_ecc_bch_24b_ena(0)|NF_ECC_SEL_ecc_bch_12b_ena(0));


 	REG_WRITE_U32(REG_ND_CMD,NF_ND_CMD_cmd(CMD_PG_WRITE_C1));
	REG_WRITE_U32(REG_CMD2,NF_CMD2_cmd2(CMD_PG_WRITE_C2));
	REG_WRITE_U32(REG_CMD3,NF_CMD3_cmd3(CMD_PG_WRITE_C3));

	REG_WRITE_U32( REG_ND_PA0,NF_ND_PA0_page_addr0(page));
	REG_WRITE_U32( REG_ND_PA1,NF_ND_PA1_page_addr1(page>>8));
	REG_WRITE_U32( REG_ND_PA2,NF_ND_PA2_addr_mode(addr_mode)|NF_ND_PA2_page_addr2(page>>16));
	REG_WRITE_U32( REG_ND_PA3,NF_ND_PA3_page_addr3((page>>21)&0x7));
	REG_WRITE_U32(REG_ND_CA0,NF_ND_CA0_col_addr0(0));
	REG_WRITE_U32(REG_ND_CA1,NF_ND_CA1_col_addr1(0));

	dma_len = this->page_size >> 9;
	REG_WRITE_U32(REG_DMA_CTL2,NF_DMA_CTL2_dma_len(dma_len));

	dram_sa = ( (uint32_t)write_page_buf >> 3);
	REG_WRITE_U32(REG_DMA_CTL1,NF_DMA_CTL1_dram_sa(dram_sa));

	if (oob_buf)
		spare_dram_sa = ( (uint32_t)nf_oob_buf >> 3);
	else
		spare_dram_sa = ( (uint32_t)nf_oob_buf>> 3);

	//patch
	RTK_FLUSH_CACHE((unsigned long) write_page_buf, this->page_size);
	RTK_FLUSH_CACHE((unsigned long) nf_oob_buf, this->oob_size);
	REG_WRITE_U32(REG_SPR_DDR_CTL,NF_SPR_DDR_CTL_spare_ddr_ena(1)|
		NF_SPR_DDR_CTL_per_2k_spr_ena(1)|NF_SPR_DDR_CTL_spare_dram_sa(spare_dram_sa));

	
	RTK_FLUSH_CACHE((unsigned long) write_page_buf, this->page_size);
	RTK_FLUSH_CACHE((unsigned long) nf_oob_buf, this->oob_size);
	REG_WRITE_U32(REG_DMA_CTL3,NF_DMA_CTL3_cp_enable(0)|NF_DMA_CTL3_cp_first(0)|NF_DMA_CTL3_ddr_wr(0)|NF_DMA_CTL3_dma_xfer(0x1));
	

	REG_WRITE_U32(REG_AUTO_TRIG,NF_AUTO_TRIG_auto_trig(1)|NF_AUTO_TRIG_spec_auto_case(0)| NF_AUTO_TRIG_auto_case(auto_trigger_mode));

	WAIT_DONE(REG_AUTO_TRIG,0x80,0);
	WAIT_DONE(REG_DMA_CTL3,0x01,0);
	REG_WRITE_U32(REG_POLL_FSTS,NF_POLL_FSTS_bit_sel(6)|NF_POLL_FSTS_trig_poll(1) );
	WAIT_DONE(REG_POLL_FSTS,0x01,0x0);
	WAIT_DONE(REG_ND_CTL,0x40,0x40);//add by alexchang 0416-2010

	if ( REG_READ_U32(REG_ND_DAT) & 0x01 ){
		NAND_ERR("write is not completed at page %d\n", page);
		return -1;
	}


	return rc;
}

static int rtk_nand_profile (struct mtd_info *rtk_mtd)
{
	int maxchips = 4;
	if (rtk_nand_scan (rtk_mtd, maxchips) < 0 || rtk_mtd->size == 0){
		NAND_ERR("Error, cannot do nand_scan(on-board)\n");
		return -ENODEV;
	}

	if ( !(rtk_mtd->oobblock&(0x200-1)) ){
		REG_WRITE_U32( REG_PAGE_LEN,rtk_mtd->oobblock >> 9);
	}else{
		NAND_ERR("Error: pagesize is not 512B Multiple");
		return -1;
	}
	return 0;
}

static void display_version (void)
{
	const char *revision;
	const char *date;
	char *running = (char *)VERSION;
	strsep(&running, " ");
	strsep(&running, " ");
	revision = strsep(&running, " ");
	date = strsep(&running, " ");
	printf(BANNER " Rev:%s (%s)\n", revision, date);
}

void endian_swap(unsigned int* x)
{
	*x = (*x>>24) |((*x<<8) & 0x00FF0000) |((*x>>8) & 0x0000FF00) |(*x<<24);
}

/*******************************
 * init NAND flash ctrler
 *******************************/
static void nf_ctrler_init(void)
{
	// setup nf controller frequency
	REG_WRITE_U32( REG_TIME_PARA1,NF_TIME_PARA1_T1(0x2));
	REG_WRITE_U32( REG_TIME_PARA2,NF_TIME_PARA2_T2(0x2));
	REG_WRITE_U32( REG_TIME_PARA3,NF_TIME_PARA3_T3(0x2));

	// setup T_WHR_ADL delay clock
	REG_WRITE_U32( REG_DELAY_CTL,NF_DELAY_CTL_T_WHR_ADL(0x09) );

	//disable edo-mode
	REG_WRITE_U32( REG_MULTI_CHNL_MODE,NF_MULTI_CHNL_MODE_edo(0));

	// disable all interrupt from nf controller
	//REG_WRITE_U32(REG_NAND_ISREN,0x1);
	REG_WRITE_U32(REG_SPR_DDR_CTL,~0x40000000);

	/*******************************
	 * reset NAND flash start
	 *******************************/
	REG_WRITE_U32( REG_PD,~(0x1 << 0) );

	// set 'reset' command
	REG_WRITE_U32( REG_ND_CMD,CMD_RESET );

	// issue command
	REG_WRITE_U32( REG_ND_CTL,0x80);

	// polling go bit to until go bit de-assert (command completed)
	WAIT_DONE(REG_ND_CTL,0x80,0);

	// check ready/busy pin of NAND flash
	WAIT_DONE(REG_ND_CTL,0x40,0x40);
}

//cpsel--->0:register, 1:CW, 2:E-Fuse
//cpmode--->//0 for CBC mode, 0x200 for ECB mode
static int rtk_nand_cp_setting(unsigned int cpsel, unsigned int cpmode, unsigned int keyindex)
{

	//sanity check
	if((cpsel >= 3)||(cpmode >=2)){
		NAND_ERR("NFCP parameters setting error!\n");
		goto cp_null;
	}

	//when cpsel select CW, must setting keyindex
	if((cpsel == 1) && (keyindex == 0) ){
		NAND_ERR("NFCP parameters setting error!\n");
		goto cp_null;
	}

	cpSel = cpsel << 0;
	u32CP_mode = cpmode << 9;
	flash_protection_root_key_index = keyindex << 2;

	//when cpsel select register, get regkey
	if(!cpSel ){
		regKey0 = (unsigned int*)&u8regKey0;
		regKey1 = (unsigned int*)&u8regKey1;
		regKey2 = (unsigned int*)&u8regKey2;
		regKey3 = (unsigned int*)&u8regKey3;
		endian_swap(regKey0);
		endian_swap(regKey1);
		endian_swap(regKey2);
		endian_swap(regKey3);
	}

	is_NF_CP_Enable_read = 1;
	is_NF_CP_Enable_write = 1;

	NAND_INFO("Enable NFCP...");

	if(cpSel == 0)
		NAND_INFO("register...");
	else if(cpsel == 1)
		NAND_INFO("CW...");
	else if(cpsel == 2)
		NAND_INFO("E-Fuse...");
	else
		NAND_INFO("Unknown...");

	if(u32CP_mode == 0)
		NAND_INFO("CBC.\n");
	else if(u32CP_mode == 0x200)
		NAND_INFO("ECB.\n");
	else
		NAND_INFO("Unknown.\n");

	return 0;

cp_null:
	printk("[%s]Disable NFCP...\n",__FUNCTION__);
	is_NF_CP_Enable_write = is_NF_CP_Enable_read = 0;
	return -1;

}

static int rtk_nand_wp_setting(unsigned int gpio_dir_base, unsigned int gpio_out_base)
{
#if  defined(CONFIG_REALTEK_PCBMGR)
	unsigned long long pin_WP_info;
#endif

	g_regGPIODirBase = gpio_dir_base;
	g_regGPIOOutBase = gpio_out_base;

#if  defined(CONFIG_REALTEK_PCBMGR)
	if(pcb_mgr_get_enum_info_byname("PIN_FLASH_WP",&pin_WP_info)!=0)
	{
		g_WP_en = 0;
		printk("PIN_FLASH_WP is not defined\n");
#if (defined(Config_Board_TV043_RENO_V001) || defined(Config_Board_TV043_RENO_V002))
		REG_WRITE_U32(0xb801bd0c, 0x00000081);
		REG_WRITE_U32(0xb801bd20, 0x00000081);
#endif
	}else{
		NAND_INFO("PIN_FLASH_WP defined value 0x%llx\n",pin_WP_info);
		g_NFWP_no = ((unsigned char) ((pin_WP_info >> 8) & 0xFF));
		g_NFWP_value_INV = ((unsigned char) ((pin_WP_info >> 24) & 0xFF));
		g_NFWP_pinType = GET_PIN_TYPE(pin_WP_info);
		g_NFWP_value_init = ((unsigned char) ((pin_WP_info >> 32) & 0xFF)); //0 enable WP;1 disable WP
		if(g_NFWP_value_init)
		{
			NAND_INFO("PIN_FLASH_WP default disable,g_NFWP_value_init = %d\n",g_NFWP_value_init);
			g_WP_en = 0;
		}else{
			NAND_INFO("PIN_FLASH_WP default enable,g_NFWP_value_init = %d\n",g_NFWP_value_init);
			g_WP_en = 1;
		}

		if(g_NFWP_value_INV==0)
		{
			g_NFWP_value_en = 0;
			g_NFWP_value_dis = 1;
		}else{
			g_NFWP_value_en = 1;
			g_NFWP_value_dis = 0;
		}

	}
#else
	NAND_INFO("Force disable FLASH_WP!!\n");
	g_WP_en=0;
#endif

	NAND_INFO("g_regGPIODirBase = 0x%x\n",g_regGPIODirBase);
	NAND_INFO("g_regGPIOOutBase = 0x%x\n",g_regGPIOOutBase);
	NAND_INFO("g_NFWP_no = %d\n",g_NFWP_no);
	NAND_INFO("g_NFWP_value_en = %d\n",g_NFWP_value_en);
	NAND_INFO("g_NFWP_value_dis = %d\n",g_NFWP_value_dis);
	NAND_INFO("g_NFWP_pinType = %d\n",g_NFWP_pinType);
	return 0;
}

void nand_chip_function_init(struct mtd_info *mtd)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	this->select_chip 		= rtk_nand_select_chip;
	this->scan_bbt 			= nf_init_table;
	this->read_id			= rtk_nand_read_id;
	this->read_ecc_page 		= rtk_read_ecc_page;
	this->read_oob 			= rtk_read_oob;
	this->write_ecc_page		= rtk_write_ecc_page;
	this->write_oob			= rtk_write_oob;
	this->erase_block		= rtk_erase_block;
	this->get_edo_mode		= rtk_nand_get_edo_mode;
	this->set_timing		= rtk_nand_set_timing;
	this->clock_select		= rtk_nand_clock_select;
	this->suspend			= NULL;
	this->resume			= NULL;
	this->sync			= NULL;
}

static int  rtk_nand_init (struct mtd_info *rtk_mtd)
{
	struct nand_chip *this = (struct nand_chip *)rtk_mtd->priv;
	int rc = 0;
	display_version();
	//init nf controler
	nf_ctrler_init();

	//print info
	if(nf_is_random_mode()){
		NAND_INFO("Nand flash : Random access mode!\n");
	}else{
		NAND_INFO("Nand flash : Sequential access mode!\n");
	}

	//default value
	g_BootcodeSize = SYS_PARAM_BOOTCODE_SIZE;
	g_Factory_param_size = SYS_PARAM_FACTORY_SIZE;
	//wp setting
	rtk_nand_wp_setting(REG_GPIO_DIR_BASE, REG_GPIO_OUT_BASE);
	disable_nand_wp();

	//for on-the-fly func
	if(CONFIG_MTD_NAND_CP_ENABLE){
		//setting CW/CBC
		rtk_nand_cp_setting(1,0,124);
	}
	/*nand chip function init*/
	nand_chip_function_init(rtk_mtd);

	/*scan nandflash get common information*/
	if( rtk_nand_profile(rtk_mtd) < 0 ){
		rc = -1;
		return rc;
	}
	/*build bbt table*/
	return this->scan_bbt(rtk_mtd);
}

int board_nand_init(struct mtd_info *mtd)
{
	int rc = 0;
	if((mtd==NULL)||(mtd->priv == NULL)){
		NAND_ERR(" error: board nand init failed!\n");
		rc = -1;
		return rc;

	}
	if((rc = rtk_nand_init (mtd)) == 0){
		NAND_INFO("Realtek Nand Flash Driver is successfully installing.\n");
	}
	return rc;
}
