/* Register */
#define REG_HW_SEMAPHORE		0xb801a000
#define REG_UART0_BASE			0xb8062300
#define REG_UART1_BASE			0xb801b100
#define DEFAULT_UART			REG_UART1_BASE

#define s0	$16
#define s1	$17

/* Value */
#define PRID_LX_5281_ACPU1		0x025ADC02
#define PRID_LX_5281_ACPU2		0x025CDC02
#define KSEG_MSK          		0xE0000000
#define KSEG0BASE				0x80000000
#define KSEG1BASE				0xA0000000
#define AUDIO_FW_ENTRY_PTR_A1	0xa00000e0
#define AUDIO_FW_ENTRY_PTR_A2	0xa000007c

#define KSEG0A(reg)     and reg, ~KSEG_MSK; or reg, KSEG0BASE

	.section ".text", "ax"
	.globl	a_entry
	.ent	a_entry
a_entry:
	.set noreorder

	/* set 5280 CP0 status register and vector interrupt address */
#if 1
	.word 0x40106000,0x3c118000,0x36310001,0x02118025,\
	     0x3c11ffbf,0x3631ffff,0x02118024,0x40906000,\
	     0x3c108000,0x36101900,0x40f01000,0x00000000,\
	     0x3c1000ff,0x40f00000,0x00000000
#else

	mfc0	s0, $12
	lui		s1,0x8000
	ori		s1,s1,0x1
	or		s0,s0,s1
	lui		s1,0xffbf
	ori		s1,s1,0xffff
	and		s0,s0,s1
	mtc0	s0,$12
	lui		s0,0x8000
	ori		s0,s0,0x1900
	mtlxc0	s0,INTVEC
	nop
	lui		s0,0xff
	mtlxc0	s0,ESTATUS
	nop

#endif


	/* Print Await */
	li		$8, 0x41	/* A */
	sw		$8, DEFAULT_UART

	li		$8, 0x77	/* w */
	sw		$8, DEFAULT_UART

	li		$8, 0x61	/* a */
	sw		$8, DEFAULT_UART

	li		$8, 0x69	/* i */
	sw		$8, DEFAULT_UART

	li		$8, 0x74	/* t */
	sw		$8, DEFAULT_UART

	li		$8, 0x20
	sw		$8, DEFAULT_UART

	/* Wait for HW semaphore */
	li		$9, KSEG1BASE
	la		$8, REG_HW_SEMAPHORE
	or		$8, $9		/* Make sure it is uncached */

get_hw_semaphore :
	# delay a while (reduce bus traffic)
	li		$10, 100
1:
	subu	$10, 0x1
	bnez	$10, 1b
	nop

	# check if HW semaphore is available
	lw		$9, 0($8)
	nop		/* 5181 need nop after lw */
	beq		$9, $0, get_hw_semaphore
	nop

a12_go:
	/***************************************/
	/* distinguish between ACPU1 and ACPU2 */
	/***************************************/
	mfc0	$17, $15
	li		$18, PRID_LX_5281_ACPU1
	bne		$17, $18, a2_go
	nop
	b		a1_go
	nop

a12_hang_loop:
	b		a12_hang_loop
	nop

a1_go:
	/* Print A1go */
	li		$8, 0x41	/* A */
	sw		$8, DEFAULT_UART

	li		$8, 0x31	/* 1 */
	sw		$8, DEFAULT_UART

	li		$8, 0x67	/* g */
	sw		$8, DEFAULT_UART

	li		$8, 0x6F	/* o */
	sw		$8, DEFAULT_UART

	li		$8, 0x20
	sw		$8, DEFAULT_UART

	li		$17, AUDIO_FW_ENTRY_PTR_A1
	li		$18, REG_HW_SEMAPHORE
	nop
	lw		$16, 0($17)
	nop
	bnez	$16, 1f
	nop
	sw		$0, ($18)
a1_loop:
	b		a1_loop
	nop
1:
	jalr	$16
	sw		$0, ($18)


a2_go:
	/* Print A2go */
	li		$8, 0x41	/* A */
	sw		$8, DEFAULT_UART

	li		$8, 0x32	/* 2 */
	sw		$8, DEFAULT_UART

	li		$8, 0x67	/* g */
	sw		$8, DEFAULT_UART

	li		$8, 0x6F	/* o */
	sw		$8, DEFAULT_UART

	li		$8, 0x20
	sw		$8, DEFAULT_UART


	li		$17, AUDIO_FW_ENTRY_PTR_A2
	li		$18, REG_HW_SEMAPHORE
	nop
	lw		$16, 0($17)
	nop
	bnez	$16, 1f
	nop
	sw		$0, ($18)
a2_loop:
	b		a2_loop
	nop
1:
	jalr	$16
	sw		$0, ($18)

/*END(a_entry)  */
	.set reorder
	.end a_entry

