
/*
 *  mcp.c
 *
 *  Implementation of hardware CP module
 *
 **/
#include <common.h>
#include <asm/arch/system.h>
#include <asm/arch/error_type.h>
#include <drivers/mcp/rtk_mcp_core.h>
#include <malloc.h>

void * MCP_MALLOC(unsigned int len, MEM_ALLOC_TYPE cache_type)
{
    unsigned char * pdata = NULL; 
    if(cache_type == MEM_ALLOC_UNCACHE) {
        pdata = (unsigned char *)malloc_noncache_align(len, 8);
    } else {
        pdata = (unsigned char *)malloc(len);
    }
    if(pdata == NULL) {
        printf("%s[%d] memory alloc failed..\n", __FUNCTION__, __LINE__);
    }
    
    return pdata;
}

void MCP_FREE(void *data, unsigned int size, MEM_ALLOC_TYPE cache_type)
{
    if(cache_type == MEM_ALLOC_UNCACHE)
        free_noncache(size); 
    else
        free(data);
}


#if 1
int AES_CBC_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4])
{
    int ret = 0;
    unsigned int cur_pos = 0;
    unsigned int cur_iv_key[4];
    unsigned int last_iv_key[4];
    while(cur_pos < length) {
        if((length - cur_pos) > MAX_CP_EN_DE_SEGMENT_SIZE) {
            unsigned char *plast_raw_iv = src_addr + cur_pos + MAX_CP_EN_DE_SEGMENT_SIZE - 16;
            last_iv_key[0]  = MCP_B4TL(plast_raw_iv[0], plast_raw_iv[1], plast_raw_iv[2], plast_raw_iv[3]);
            last_iv_key[1]  = MCP_B4TL(plast_raw_iv[4], plast_raw_iv[5], plast_raw_iv[6], plast_raw_iv[7]);
            last_iv_key[2]  = MCP_B4TL(plast_raw_iv[8], plast_raw_iv[9], plast_raw_iv[10], plast_raw_iv[11]);
            last_iv_key[3]  = MCP_B4TL(plast_raw_iv[12], plast_raw_iv[13], plast_raw_iv[14], plast_raw_iv[15]);
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, cur_iv_key);
            if(ret != 0)
                break;
            cur_pos += MAX_CP_EN_DE_SEGMENT_SIZE;
            memcpy(cur_iv_key, last_iv_key, 16);
        } else {
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, length - cur_pos, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, length - cur_pos, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, cur_iv_key);
            cur_pos = length;/*no need to do this step*/
            break;
        }
    }
    return ret;

}

int AES_CBC_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4])
{
    int ret = 0;
    unsigned int cur_pos = 0;
    unsigned int cur_iv_key[4];
    unsigned int last_iv_key[4];
    memcpy(cur_iv_key, ini_key, 16);
    while(cur_pos < length) {
        if((length - cur_pos) > MAX_CP_EN_DE_SEGMENT_SIZE) {
            unsigned char *plast_raw_iv = dst_addr + cur_pos + MAX_CP_EN_DE_SEGMENT_SIZE - 16;
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, cur_iv_key);
            if(ret != 0)
                break;
            cur_pos += MAX_CP_EN_DE_SEGMENT_SIZE;

            last_iv_key[0] = (plast_raw_iv[0] << 24) + (plast_raw_iv[1] << 16) + (plast_raw_iv[2] << 8) + plast_raw_iv[3];
            last_iv_key[1] = (plast_raw_iv[4] << 24) + (plast_raw_iv[5] << 16) + (plast_raw_iv[6] << 8) + plast_raw_iv[7];
            last_iv_key[2] = (plast_raw_iv[8] << 24) + (plast_raw_iv[9] << 16) + (plast_raw_iv[10] << 8) + plast_raw_iv[11];
            last_iv_key[3] = (plast_raw_iv[12] << 24) + (plast_raw_iv[13] << 16) + (plast_raw_iv[14] << 8) + plast_raw_iv[15];
            memcpy(cur_iv_key, last_iv_key, 16);

        } else {
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, length - cur_pos, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, 
                                (unsigned int)MCP_KEY_FROM_CW, src_addr + cur_pos, length - cur_pos, 
                                dst_addr + cur_pos, (unsigned int *)key_entry, cur_iv_key);
            cur_pos = length;
            break;
        }
    }
    return ret;
}

int AES_ECB_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry)
{
    return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, (unsigned int *)key_entry, NULL);
}

int AES_ECB_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry)
{
    return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, (unsigned int *)key_entry, NULL);
}

#endif
int DES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int DES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int DES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{    
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int DES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int TDES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int TDES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int TDES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int TDES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

//=================================================================================================================================================
int AES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int AES_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}


int AES_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int SHA1_hash(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int iv[5])
{
    if(iv != NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA1, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, iv, iv + 4);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA1, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, NULL, NULL);
}

int SHA256_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[8])
{
    int ret = 0;
    unsigned int cur_pos = 0;
    unsigned int cur_iv_key[8] = {0};
    if ((src_addr == NULL) || (dst_addr == NULL))
        return ERR_INVALID_PARAM;
    if(iv != NULL)
        memcpy(cur_iv_key, iv, 32);
    while(cur_pos < length) {
        if((length - cur_pos) > MAX_CP_SHA_SEGMENT_SIZE) {
            if(cur_pos != 0 || iv != NULL)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr + cur_pos, MAX_CP_SHA_SEGMENT_SIZE, dst_addr, &cur_iv_key[0], &cur_iv_key[4]);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr + cur_pos, MAX_CP_SHA_SEGMENT_SIZE, dst_addr, NULL, NULL);

            if(ret != 0)
                break;
            cur_pos += MAX_CP_SHA_SEGMENT_SIZE;
            cur_iv_key[0]  = MCP_B4TL(dst_addr[0], dst_addr[1], dst_addr[2], dst_addr[3]);
            cur_iv_key[1]  = MCP_B4TL(dst_addr[4], dst_addr[5], dst_addr[6], dst_addr[7]);
            cur_iv_key[2]  = MCP_B4TL(dst_addr[8], dst_addr[9], dst_addr[10], dst_addr[11]);
            cur_iv_key[3]  = MCP_B4TL(dst_addr[12], dst_addr[13], dst_addr[14], dst_addr[15]);
            cur_iv_key[4]  = MCP_B4TL(dst_addr[16], dst_addr[17], dst_addr[18], dst_addr[19]);
            cur_iv_key[5]  = MCP_B4TL(dst_addr[20], dst_addr[21], dst_addr[22], dst_addr[23]);
            cur_iv_key[6]  = MCP_B4TL(dst_addr[24], dst_addr[25], dst_addr[26], dst_addr[27]);
            cur_iv_key[7]  = MCP_B4TL(dst_addr[28], dst_addr[29], dst_addr[30], dst_addr[31]);

        } else {
            if(cur_pos != 0 || iv != NULL)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr + cur_pos, length - cur_pos, dst_addr, &cur_iv_key[0], &cur_iv_key[4]);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr + cur_pos, length - cur_pos, dst_addr, NULL, NULL);
            cur_pos = length;/*no need to do this step*/
            break;
        }
    }
    return ret;
}

////
int CAM_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int CAM_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}


int CAM_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, src_addr, length, dst_addr, key, ini_key);
}

int AES_hash(unsigned char * src_addr, int length, unsigned char * dst_addr, unsigned int block_size)
{
    int res;
    t_mcp_descriptor dscpt;
    unsigned int *init_key = NULL;
    if ((src_addr == NULL) || (dst_addr == NULL))
        return -1;
    if ((length <= 0) || (block_size <= 0))
        return -1;

    while(length > 0) {
        unsigned int cur_size = length < block_size ? length : block_size;
        length -= cur_size;
        res = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_H, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, src_addr, cur_size, dst_addr, NULL, init_key);
        if(res)
            break;
        if (length > 0) {
            // update descriptor for next block
            update_iv(&dscpt, (unsigned char *)dst_addr, 1);
            init_key = dscpt.ini_key;
            src_addr += cur_size;
        } else {
            break;
        }
    }
    return res;
}


int SHA1_padding(char* p_buff, unsigned long buff_len, const unsigned int data_len)
{
    unsigned long long bit_len = ((unsigned long long) data_len) << 3;
    int pad_sz = 64 - (data_len & 0x3f);

    if (pad_sz <= 8)
        pad_sz += 64;

    if (buff_len < pad_sz) {
        printf("padding SHA1 data failed, no enough space left for data padding. pad_sz = %d, remain_sz = %ld\n", pad_sz, buff_len);
        return -1;
    }

    memset(p_buff, 0, pad_sz);

    *p_buff |= 0x80;

    // pad bit length in big endian format
    p_buff   += pad_sz - 8;
    *p_buff++ = (bit_len >> 56) & 0xFF;
    *p_buff++ = (bit_len >> 48) & 0xFF;
    *p_buff++ = (bit_len >> 40) & 0xFF;
    *p_buff++ = (bit_len >> 32) & 0xFF;
    *p_buff++ = (bit_len >> 24) & 0xFF;
    *p_buff++ = (bit_len >> 16) & 0xFF;
    *p_buff++ = (bit_len >>  8) & 0xFF;
    *p_buff++ = (bit_len)       & 0xFF;

    return pad_sz;
}

int SHA256_padding(char *p_buff, const unsigned int length)
{
    unsigned long long bit_len;
    int pad_len;

    if ((length == 0) || (p_buff == NULL)) {
        return -1;
    }

    // calculate padding length
    pad_len = 64 - (length & 0x3f);
    if (pad_len <= 8)
        pad_len += 64;

    bit_len = (unsigned long long)(length << 3);

    // fill padding pattern
    memset(p_buff, 0, pad_len);
    *p_buff = 0x80;

    // pad bit length in big endian format
    p_buff   += pad_len - 8;
    *p_buff++ = (bit_len >> 56) & 0xFF;
    *p_buff++ = (bit_len >> 48) & 0xFF;
    *p_buff++ = (bit_len >> 40) & 0xFF;
    *p_buff++ = (bit_len >> 32) & 0xFF;
    *p_buff++ = (bit_len >> 24) & 0xFF;
    *p_buff++ = (bit_len >> 16) & 0xFF;
    *p_buff++ = (bit_len >>  8) & 0xFF;
    *p_buff++ = (bit_len)       & 0xFF;

    return pad_len;
}

/*======================================================================
 * Func : MCP_HMAC-SHA256
 *
 * Desc : Generating message authentication code via HMAC-SHA256
 *
 * Parm : keyIn         : the key msg provider and you are owner.
 *        keyIn_len     : the length of key
 *        msgIn         : the msg need to verify whether correct or not
 *        msgIn_len     : the length of msg
 *        p_out         : the msg authertication code
 *
 * Retn : >=0 : success
 *        < 0 : failed,
 *======================================================================*/
/* +-----------------------+---------------------+                 +-----------------+
 * |inner_key(key^0x36)64B | Message(msg)        | Do SHA-256 Hash | First_Hash(32B) |
 * +-----------------------+---------------------+                 +-----------------+
 *                                __________________________________________|
 *                                |
 * +-----------------------+-----------------+                 +------------------------+
 * |outer_key(key^0x5c)64B | First_Hash(32B) | Do SHA-256 Hash | Second_Hash(p_out)32B  |
 * +-----------------------+-----------------+                 +------------------------+
 */
int MCP_HMAC_SHA256( unsigned char* keyIn,
                     unsigned int keyIn_len,
                     unsigned char* msg,
                     unsigned int msg_len,
                     unsigned char* p_out )
{
    char*  tmp = NULL;
    unsigned char   k_ipad[MCP_SHA256_BLOCK_SIZE] = {0}; /* inner padding - key XORd with ipad */
    unsigned char   k_opad[MCP_SHA256_BLOCK_SIZE] = {0}; /* outer padding - key XORd with opad */
    unsigned char   key[MCP_SHA256_BLOCK_SIZE] = {0};
    unsigned char   hash[MCP_HMAC_SHA256_SIZE] = {0};

    int i;
    unsigned int    error = 0;
    unsigned int    pad_sz = 0;

    tmp = malloc(MCP_SHA256_BLOCK_SIZE + msg_len + 72);

    /* 1.0 if key is longer than 64 bytes reset it to key=SAH256(key), new key is Ko*/
    if (keyIn_len > MCP_SHA256_BLOCK_SIZE) {
        pad_sz += SHA256_padding((char *)&keyIn[keyIn_len], keyIn_len);

//        DCACHE_FLUSH();

        error = SHA256_hash((unsigned char *)PHYS(keyIn), keyIn_len + pad_sz, (unsigned char *)PHYS(key), NULL);
        if( error ) {
            printf("[ERR] %s: caculate hash256 fail(%d)\n", __FUNCTION__, error );
            return 1;
        }

        keyIn_len = MCP_SHA256_BLOCK_SIZE;
    } else {
        memcpy( key, keyIn, keyIn_len );
        keyIn_len = MCP_SHA256_BLOCK_SIZE;
    }

    /* 2. XOR key with ipad and opad values */
    for ( i = 0; i < MCP_SHA256_BLOCK_SIZE; i++ ) {
        k_ipad[i] = 0x36 ^ key[i];
        k_opad[i] = 0x5c ^ key[i];
    }

    /* 3. First-Hash = H(Ko XOR Ipad(0x36) || (data to auth)) */
    memcpy(tmp, k_ipad, keyIn_len);
    memcpy(tmp + keyIn_len, msg, msg_len);
    pad_sz = SHA256_padding((char *)&tmp[keyIn_len + msg_len], keyIn_len + msg_len);

    DCACHE_FLUSH();

    error = SHA256_hash((unsigned char *)PHYS(tmp), keyIn_len + msg_len + pad_sz, (unsigned char *)PHYS(hash), NULL);
    if( error ) {
        printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
        return 1;
    }

    /* 4. Second-Hash = H(Ko XOR Opad(0x5c) || First-Hash) */
    memcpy(tmp, k_opad, keyIn_len);
    memcpy(tmp + keyIn_len, hash, 32);
    pad_sz = SHA256_padding((char *)&tmp[keyIn_len + 32], keyIn_len + 32);

    DCACHE_FLUSH();

    error = SHA256_hash((unsigned char *)PHYS(tmp), keyIn_len + 32 + pad_sz, (unsigned char *)PHYS(p_out), NULL);
    if( error ) {
        printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
        return 1;
    }
    free(tmp);
    return 0;
}

void reverse_signature( unsigned char * pSignature )
{
    unsigned char temp;
    unsigned int i;

    for( i = 0; i < 128; i ++ )  {
        temp = pSignature[i];
        pSignature[i] = pSignature[255 - i];
        pSignature[255 - i] = temp;
    }
}
