/*
 *
 *  rtk_mcp_hw.c
 *
 *  Implementation of hardware CP module
 *
 *
 **/

#include <common.h>
#include <asm/arch/system.h>
#include <asm/arch/error_type.h>
#include <drivers/mcp/rtk_rcp_reg.h>
#include <drivers/mcp/rtk_rcp_defines.h>
#include <drivers/mcp/rtk_rcp_dbg.h>
#include <drivers/mcp/rtk_rcp.h>
static void rtk_rcp_set_reg_array(RCP_REG_TYPE reg_type, unsigned int nregs,
                                  unsigned char *data_in, unsigned int len)
{
    unsigned int i = 0;
    if(nregs > 4)
        nregs = 4;
    if(data_in == NULL || len == 0)
        return;
    switch(reg_type) {
        case RCP_REG_TYPE_KEY:
            for(i = 0; i < nregs; i++)
                SET_CP_REG_MOD_KEY(i, RCP_BS4TL(data_in + 4 * i));
            break;
        case RCP_REG_TYPE_KEY1:
            for(i = 0; i < nregs; i++)
                SET_CP_REG_MOD_KEY1(i, RCP_BS4TL(data_in + 4 * i));
            break;    
        case RCP_REG_TYPE_IV:
            for(i = 0; i < nregs; i++)
                SET_CP_REG_MOD_IV(i, RCP_BS4TL(data_in + 4 * i));
            break;
        case RCP_REG_TYPE_DATAIN:
            for(i = 0; i < nregs; i++)
                SET_CP_REG_MOD_DATAIN(i, RCP_BS4TL(data_in + 4 * i));
            break;
        case RCP_REG_TYPE_DATAOUT:
            for(i = 0; i < nregs; i++)
                SET_CP_REG_MOD_DATAOUT(i, RCP_BS4TL(data_in + 4 * i));
            break;
        default:
            break;
    }
}


static void rtk_rcp_get_reg_array(RCP_REG_TYPE reg_type, unsigned int nregs,
                           unsigned char *data_out, unsigned int len)
{
    unsigned int i = 0;
    if(data_out == NULL || len == 0)
        return;
    if(nregs > 4)
        nregs = 4;
    switch(reg_type) {
        case RCP_REG_TYPE_KEY:
            for(i = 0; i < nregs; i++)
                RCP_LTBS4(data_out + 4 * i, GET_CP_REG_MOD_KEY(i));
            break;
        case RCP_REG_TYPE_IV:
            for(i = 0; i < nregs; i++)
                RCP_LTBS4(data_out + 4 * i, GET_CP_REG_MOD_IV(i));
            break;
        case RCP_REG_TYPE_DATAIN:
            for(i = 0; i < nregs; i++)
                RCP_LTBS4(data_out + 4 * i, GET_CP_REG_MOD_DATAIN(i));
            break;
        case RCP_REG_TYPE_DATAOUT:
            for(i = 0; i < nregs; i++)
                RCP_LTBS4(data_out + 4 * i, GET_CP_REG_MOD_DATAOUT(i));
            break;
        default:
            break;
    }
}



/*======================================================================
 * Func : RTK_RCP_SET_MODE
 *
 * Desc :
 *
 * Parm : N/A
 *
 * Retn : NULL / data pointer
 *======================================================================*/
static unsigned char* RTK_RCP_SET_MODE(
    int                     Mode,
    unsigned char*          pData,
    unsigned long           DataLen,
    unsigned int            BlockSize,
    unsigned char*          pKey
)
{
    switch(Mode) {
        case KEY_MODE:
            if (pData == NULL) {
                SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_KEY_MODE_OTP);
            } else if ((unsigned long)pData <= MAX_CW_ENTRY) {
                SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_KEY_MODE_CW);
                SET_CP_REG_MOD_SET2(GET_CP_REG_MOD_SET2() | REG_SET2_KEY_ENTRY(((unsigned long)pData) - 1));// each cw takes 8 entry
            } else {
                unsigned char tmp_key[16] = {0};
                if (DataLen == 16) {
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY, 4, pData, 16);
                } else if (DataLen == 7) {   // DES Key
                    tmp_key[0] = 0;
                    memcpy(tmp_key + 1, pData, 7);
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY, 2, tmp_key, 8);

                } else if (DataLen == 21) { // TDES Key
                    tmp_key[0] = 0;
                    tmp_key[8] = 0;
                    memcpy(tmp_key + 1, pData, 7);      // store K1, K2, K3 should be stored in Data2,3
                    memcpy(tmp_key + 9, pData + 7, 7);
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY, 4, tmp_key, 16);
                } else if (DataLen == 24) { 
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY, 4, pData, 16);
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY1, 2, pData + 16, 8);
                } else if (DataLen == 32) { 
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY, 4, pData, 16);
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_KEY1, 4, pData + 16, 16);
                }else {
                    return NULL;
                }
            }
            break;

        case IV_MODE:
            if(pData != NULL) {
                if ((unsigned long)pData <= MAX_CW_ENTRY) {
                    SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_IV_MODE_CW);
                    SET_CP_REG_MOD_SET2(GET_CP_REG_MOD_SET2() | REG_SET2_IV_ENTRY(((unsigned long)pData) - 1));// each cw takes 8 entry
                } else {
                    SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_IV_MODE_REG);
                    if (DataLen == 16) {    // 128 bits IV
                        rtk_rcp_set_reg_array(RCP_REG_TYPE_IV, 4, pData, 16);

                    } else if (DataLen == 8) { // 64 bits IV
                        rtk_rcp_set_reg_array(RCP_REG_TYPE_IV, 2, pData, 8);
                    }
                }
            }
            break;

        case INPUT_MODE:
            if ((unsigned long)pData <= MAX_CW_ENTRY) {
                if (((unsigned long)pData) + (DataLen >> 3) > MAX_CW_ENTRY) {
                    RCP_WARNING("Input from CW, but out of range\n");
                    return NULL;
                }
                SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_INPUT_MODE_CW);
            }
            break;

        case OUTPUT_MODE:

            if ((unsigned long)pData <= MAX_CW_ENTRY) {
                if (((unsigned long)pData) + (DataLen >> 3) > MAX_CW_ENTRY) {
                    RCP_WARNING("Output to CW, but out of range\n");
                    return NULL;
                }
                SET_CP_REG_MOD_SET1(GET_CP_REG_MOD_SET1() | REG_SET1_OUTPUT_MODE_CW);
            }
            break;

        case INPUT_ENTRY:

            if ((unsigned long)pData <= MAX_CW_ENTRY) {
                unsigned char tmp_data[16] = {0};
                SET_CP_REG_MOD_SET2((GET_CP_REG_MOD_SET2() & 0xff00ffff) | REG_SET2_INPUT_ENTRY(((unsigned long)pData) - 1));
                rtk_rcp_set_reg_array(RCP_REG_TYPE_DATAIN, 4, tmp_data, 16);
                pData += BlockSize / 8;       // jump 2 cw entry...
            } else {
                if (BlockSize == 8) {
                    unsigned char tmp_data[16] = {0};
                    memcpy(tmp_data, pData, 8);
                    if(pKey != NULL) {       // TDES
                        // for TDES only. TDES use Data2,Data3 to store K3
                        tmp_data[8] = 0;
                        memcpy(tmp_data + 9, pKey + 14, 7);
                    }
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_DATAIN, 4, tmp_data, 16);
                } else {
                    rtk_rcp_set_reg_array(RCP_REG_TYPE_DATAIN, 4, pData, 16);
                }

                pData += BlockSize;
            }
            break;

        case OUTPUT_ENTRY:

            if ((unsigned long)pData <= MAX_CW_ENTRY) {
                unsigned char tmp[16] = {0};
                SET_CP_REG_MOD_SET2((GET_CP_REG_MOD_SET2() & 0xffffff) | REG_SET2_OUTPUT_ENTRY((unsigned long)pData - 1));
                rtk_rcp_get_reg_array(RCP_REG_TYPE_DATAOUT, 4, tmp, 16);
                pData += BlockSize / 8;
            } else {
                if (BlockSize == 16) {
                    rtk_rcp_get_reg_array(RCP_REG_TYPE_DATAOUT, 4, pData, 16);
                } else if (BlockSize == 8) {
                    rtk_rcp_get_reg_array(RCP_REG_TYPE_DATAOUT, 2, pData, 8);
                } else {
                    return NULL;
                }
                pData += BlockSize;
            }
            break;
        default:
            break;
    }

    return pData;
}



static int RTK_RCP_CRYPTION_INIT(unsigned int rcp_crypt_set, unsigned char *key, unsigned int key_len, 
                             unsigned char *iv, unsigned int iv_len, unsigned char* pDataIn,
                             unsigned char* pDataOut, unsigned long DataLen)
{
    SET_CP_REG_MOD_SET(rcp_crypt_set);
    SET_CP_REG_MOD_SET1(0);
    SET_CP_REG_MOD_SET2(0);
    RTK_RCP_SET_MODE(KEY_MODE, key, key_len, 0, NULL);
    if(iv != NULL)
        RTK_RCP_SET_MODE(IV_MODE, iv, iv_len, 0, NULL);
    RTK_RCP_SET_MODE(INPUT_MODE, pDataIn, DataLen, 0, NULL);
    RTK_RCP_SET_MODE(OUTPUT_MODE, pDataOut, DataLen, 0, NULL);
    return 0;
}


static int RTK_RCP_DO_CRYPTION(unsigned int rcp_crypt_set, unsigned char *Key, unsigned int BlockSize, 
                                        unsigned char* pDataIn, unsigned char* pDataOut, unsigned long DataLen)
{
    unsigned long  out_len = 0;
    
    while(DataLen >= BlockSize) {
        if ((unsigned long)pDataOut <= MAX_CW_ENTRY) {
            SET_CP_REG_MOD_SET2((GET_CP_REG_MOD_SET2() & 0xffffff) |
                                REG_SET2_OUTPUT_ENTRY((unsigned long)pDataOut - 1));
            pDataOut += BlockSize / 8;
            if(!(pDataIn = RTK_RCP_SET_MODE(INPUT_ENTRY, pDataIn, DataLen, BlockSize, Key)))
                goto error_occur;
        } else {
            if(!(pDataIn = RTK_RCP_SET_MODE(INPUT_ENTRY, pDataIn, DataLen, BlockSize, Key)))
                goto error_occur;
            rtk_rcp_get_reg_array(RCP_REG_TYPE_DATAOUT, BlockSize/sizeof(unsigned int), pDataOut, BlockSize);
            pDataOut += BlockSize;
        }

        DataLen -= BlockSize;
        out_len += BlockSize;
        if(rcp_crypt_set & REG_SET_FIRST_128)
            SET_CP_REG_MOD_SET(GET_CP_REG_MOD_SET() & (~REG_SET_FIRST_128));
    }
    return out_len;
error_occur:
    return 0;
}


int RTK_RCP_AES_CBC_Cipher(
    unsigned char           EnDe,
    unsigned char           Key[16],
    unsigned char           IV[16],
    unsigned char*          pDataIn,
    unsigned char*          pDataOut,
    unsigned long           DataLen
)
{
    unsigned long  out_len = 0;


    if (pDataIn == NULL || pDataOut == NULL || IV == NULL) {
        printf("Run RTK_RCP_AES_CBC_Cipher failed, IV should not be NULL \n");
        return 0;
    }

    RTK_RCP_CRYPTION_INIT((EnDe) ? RCP_AES_128_CBC_ENC : RCP_AES_128_CBC_DEC,
            Key, 16, IV, 16, pDataIn, pDataOut, DataLen);
    out_len = RTK_RCP_DO_CRYPTION((EnDe) ? RCP_AES_128_CBC_ENC : RCP_AES_128_CBC_DEC, NULL, 16,
                                            pDataIn, pDataOut, DataLen);
    return out_len;
}


static int RTK_RCP_CW_LOGIC_OP(RCP_CW_LOGIC_MODE_TYPE cw_logic_mode, unsigned int CwIn1, unsigned int CwIn2, unsigned int CwOut)
{
    unsigned char tmp_data[16];
    if(CwIn1 > MAX_CW_ENTRY || CwIn2 > MAX_CW_ENTRY || CwOut > MAX_CW_ENTRY)
        return 0;

    switch(cw_logic_mode) {
        case CW_LOGIC_MODE_AND:
            SET_CP_REG_MOD_SET(RCP_CW_LOGIC_OPSEL_AND);
            break;
        case CW_LOGIC_MODE_OR:
            SET_CP_REG_MOD_SET(RCP_CW_LOGIC_OPSEL_OR);
            break;
        case CW_LOGIC_MODE_XOR:
            SET_CP_REG_MOD_SET(RCP_CW_LOGIC_OPSEL_XOR);
            break;
        default:
            return 0;
    }
    SET_CP_REG_MOD_SET1(REG_SET1_KEY_MODE_CW | REG_SET1_IV_MODE_CW | REG_SET1_OUTPUT_MODE_CW);
    SET_CP_REG_MOD_SET2(REG_SET2_KEY_ENTRY(CwIn1 - 1) | REG_SET2_IV_ENTRY(CwIn2 - 1) | REG_SET2_OUTPUT_ENTRY(CwOut - 1));
    SET_CP_REG_MOD_DATAIN(3, 0);// triger computation
    rtk_rcp_get_reg_array(RCP_REG_TYPE_DATAOUT, 1, tmp_data, 1);                // wait complete

    return 1;
}

int RTK_RCP_CW_XOR(unsigned int CwIn1, unsigned int CwIn2, unsigned int CwOut)
{
    return RTK_RCP_CW_LOGIC_OP(CW_LOGIC_MODE_XOR, CwIn1, CwIn2, CwOut);
}