#include <common.h>
#include <command.h>
#include <io.h>
#include <drivers/pwm/pwm_menu.h>

pwm_menu_state pwm_state;

char pwm_cmd_buf[256];

pwm_setmenu pwm_top_menu[] =
{
    {"ROOT", "********PWM Main menu ********", NULL, NULL}, //title
    {"LIST", "SHOW PWM LIST",go_pwm_list, NULL},
    {"DUTY", "PWM DUTY TEST", go_duty_top, NULL},
    {"FREQ", "PWM FREQ TEST", go_frequency_top, NULL},
    {"GPIO_MODE", "PWM GPIO MODE TEST", go_gpio_mode_top, NULL},
    {"STATUS", "SHOW PWM STATUS", go_show_all_info_top, NULL},
    {"PCB_INFO", "SHOW PWM PCB INFO", go_show_pwm_pcb_info_top, NULL},
    {"Exit", "go exit", go_pwm_up_menu, NULL},
    //#ifdef CONFIG_REALTEK_PWM_ROKU
    #if 0
    {"ROKU test", "GO ROKU TEST", roku_test, NULL},
    #endif
    {NULL, NULL, NULL, NULL},
};

void pwm_prepare_go_ahead(void)
{
    if(pwm_state.level > MENU_LEVEL_MAX)
        return;

    snprintf(pwm_state.path[pwm_state.level],MENU_NAME_LEN,pwm_state.menu[0].name);
    pwm_state.level++;
}

void pwm_prepare_go_back(void)
{
    if(pwm_state.level > 0)
    {
        pwm_state.level--;
    }
}

int go_pwm_list(void)
{
    rtk_pwm_list();
    return 0;
}

int get_pwm_func_param(char *name,RTK_PWM_TYPE *type, int *index, int *val)
{
    int select = 0;
    char *pin_name = NULL;
    char* buf = pwm_cmd_buf;
    rtk_pwm_list();
    printf("select pwm pin:");
    gets(buf);
    printf("\n");
    select = simple_strtoul(buf, NULL, 10);
    pin_name = rtk_pwm_name_get(select);
    if(pin_name == NULL)
        return (-1);

    if(rtk_pwm_gpio_mode_check(pin_name)<0){
        printf("Setting %s failed, it's GPIO mode, please set PWM mode first!!\n",name);
        return(-1);
    }

    (*type) = rtk_pwm_type_get(select);
    if((*type) == PWM_UNKNOW)
        return (-1);

    (*index) = rtk_pwm_index_get(*type,select);
    if((*index) < 0)
        return (-1);

    if(strcmp(name,"show_all_info")==0)
        return 0;

    printf("set pwm %s val:",name);
    gets(buf);
    printf("\n");
    (*val) = simple_strtoul(buf, NULL, 10);

    return 0;
}

int go_duty_top(void)
{
    int set_value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    if(get_pwm_func_param("duty", &pwm_type, &index, &set_value)<0)
        return (-1);

    if(set_pwm_duty(pwm_type,index,set_value) < 0)
        return (-1);

    return 0;
}

int go_frequency_top(void)
{
    int set_value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    if(get_pwm_func_param("freq", &pwm_type, &index, &set_value)<0)
        return (-1);

    if(set_pwm_frequency(pwm_type,index,set_value)<0)
        return (-1);

    return 0;
}

int go_output_top(void)
{
    int set_value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    if(get_pwm_func_param("output", &pwm_type, &index, &set_value)<0)
        return (-1);

    if(set_pwm_enable(pwm_type,index,set_value)<0)
        return (-1);

    return 0;
}

int go_polarity_top(void)
{
    int set_value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    if(get_pwm_func_param("polarity", &pwm_type, &index, &set_value)<0)
        return (-1);

    if(set_pwm_polarity(pwm_type,index,set_value)<0)
        return (-1);

    return 0;
}

int go_vsync_top(void)
{
    int set_value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    if(get_pwm_func_param("vsync", &pwm_type, &index, &set_value)<0)
        return (-1);

    if(set_pwm_sync(pwm_type,index,set_value)<0)
        return (-1);

    return 0;
}

int go_gpio_mode_top(void)
{
    int select = 0;
    int set_value = 0;
    int index = 0;
    int mode = 0;
    char *pin_name = NULL;
    char* buf = pwm_cmd_buf;
    RTK_PWM_TYPE pwm_type = -1;
    rtk_pwm_list();
    printf("select pwm pin:");
    gets(buf);
    printf("\n");
    select = simple_strtoul(buf, NULL, 10);
    pin_name = rtk_pwm_name_get(select);
    if(pin_name == NULL)
        return (-1);

    pwm_type = rtk_pwm_type_get(select);
    if(pwm_type == PWM_UNKNOW)
        return (-1);

    index = rtk_pwm_index_get(pwm_type,select);
    if(index < 0)
        return (-1);

    printf("set pwm %s mode(0:PWM_MODE 1:GPIO_MODE):",pin_name);
    gets(buf);
    printf("\n");
    mode = simple_strtoul(buf, NULL, 10);

    if(mode == GPIO_MODE){
        printf("set pwm %s val (1:high/0:low):",pin_name);
        gets(buf);
        printf("\n");
        set_value = simple_strtoul(buf, NULL, 10);
    }

    if(set_pwm_gpio_mode(pin_name,mode,set_value)<0)
        return (-1);

    return 0;
}

int go_show_pwm_pcb_info_top(void)
{
    rtk_pwm_info_dump();
    return 0;
}

int go_show_all_info_top(void)
{
    int gpio_mode = 0;
    int value = 0;
    int index = 0;
    RTK_PWM_TYPE pwm_type = -1;
    int select = 0;
    char *pin_name = NULL;
    char* buf = pwm_cmd_buf;
    rtk_pwm_list();
    printf("select pwm pin:");
    gets(buf);
    printf("\n");
    select = simple_strtoul(buf, NULL, 10);
    pin_name = rtk_pwm_name_get(select);
    if(pin_name == NULL)
        return (-1);

    pwm_type = rtk_pwm_type_get(select);
    if(pwm_type == PWM_UNKNOW)
        return (-1);

    index = rtk_pwm_index_get(pwm_type,select);
    if(index < 0)
        return (-1);

    get_pwm_duty(pwm_type,index);
    get_pwm_frequency(pwm_type,index);
    get_pwm_enable(pwm_type,index);
    get_pwm_sync(pwm_type,index);
    get_pwm_polarity(pwm_type,index);
    printf("========================\n");
    get_pwm_gpio_mode(pin_name,&gpio_mode,&value);
    printf("========================\n");
    return 0;
}

void show_pwm_menu(pwm_setmenu * p)
{
    int i = 0;

    printf("%s\n", p[0].desc);//print title
    i = 1;
    while(p[i].name != NULL)//print items
    {
        printf("[%d] %-16s : %s\n", i, p[i].name, p[i].desc);
        i++;
    }
}

void show_pwm_now_path(void)
{
    int i = 0;
    printf(FYELLOW_START);
    for(i = 0;i < pwm_state.level;++i)
    {
        if(i != 0)
        {
            printf(" -> ");
        }
        else
        {
            printf("menu path:");
        }
        printf("%s",pwm_state.path[i]);
    }
    printf("\n" FCOLOR_END);
}

int get_pwm_menu_len(pwm_setmenu * p)
{
    int i = 0;
    while(p[i].name != NULL)//print items
    {
        i++;
    }

    return i-1;//don't count p[0](p[0] is a title item)
}

int pwm_menu_wait(pwm_setmenu * p)
{
    int index = 0;
    char buf[64] = {0};
    int len;

    show_pwm_now_path();
    show_pwm_menu(p);
    len = get_pwm_menu_len(p);

    index = 0;
    while((index <= 0) || (index > len))
    {
        printf("\ninput chooice('p'- back):");
        gets(buf);
        //dump_buffer_data(buf,4);
        if(buf[0] == 'p') //if use 'p', go back
        {
            printf("\n");
            go_pwm_up_menu();
            break;
        }

        index = simple_strtoul(buf, NULL, 16);
        printf("\n");
        //printf("\nget chooice:%d\n", index);
    }

    if(p[index].func != NULL)
    {
        (*(p[index].func))();
    }
    return 0;
}

int go_pwm_up_menu(void)
{
    pwm_prepare_go_back();
    pwm_state.menu = pwm_state.menu[0].father;

    return 0;
}

//#ifdef CONFIG_REALTEK_PWM_ROKU
#if 0
int roku_test(void)
{
    char gpio_cmd_buf[256];
    char* buf = gpio_cmd_buf;
    printf("select test function:\n");
    printf("1 for DutyCycle, 2 for Period, 3 for PWM_Enable:\n");
    printf("4 for PWM_Polarity \n");

    gets(buf);
    printf("\n");
    if(buf[0] == '1') {
        printf("\n");
        printf("input DutyCycle val 0-100:");
        gets(buf);
        int val =simple_strtol(buf, NULL, 0);
        PAL_PM_PWM_DutyCycle(val);
    } else if(buf[0] == '2') {
        printf("\n");
        printf("input PWM_Period val:");
        gets(buf);
        int val =simple_strtol(buf, NULL, 0);
        PAL_PM_PWM_Period(val);
    } else if(buf[0] == '3'){
        printf("\n");
        PAL_PM_PWM_Enable();
    } else if(buf[0] == '4'){
        printf("\n");
        /*
         * Sets the polarity of the PWM.
         *      1: Sets the polarity as "normal"
         *      0: Sets the polarity as "inversed"
         */
        printf("input PWM_Polarity val 1 normal,0 inversed:");
        gets(buf);
        int val =simple_strtol(buf, NULL, 0);
        PAL_PM_PWM_Polarity(val);
    } else {
        printf("ERROR input ,range is 1-4\n");
    }

    return 0;
}
#endif

int go_menu_exit(void)
{
    printf("******** pwm menu exit! ********\n");
    return 0;
}

int go_pwm_main_menu(void)
{
    pwm_state.menu = pwm_top_menu;
    pwm_prepare_go_ahead();
    rtk_pwm_pcb_info_init();
    while(pwm_state.menu != NULL)
    {
        pwm_menu_wait(pwm_state.menu);
    }
    go_menu_exit();

    return 0;
}
