/************************************************************************
 *
 *  rtk_mcp_core.h
 *
 *  Public header file for CP module
 *
 ************************************************************************/

#ifndef __RTK_MCP_CORE_H__
#define __RTK_MCP_CORE_H__

#include "rtk_mcp_hw.h"

#define MAX_CP_EN_DE_SEGMENT_SIZE   0x00FFFFF0  /*16Mbytes - 16bytes*/
#define MAX_CP_SHA_SEGMENT_SIZE       0x00FFFFC0  /*16Mbytes - 64bytes*/
#define MCP_MAX_CW_ENTRY     128

typedef enum _MEM_ALLOC_TYPE
{
    MEM_ALLOC_CACHE = 0,
    MEM_ALLOC_UNCACHE,
}MEM_ALLOC_TYPE;

#define DEFUALT_MEM_ALLOC_TYPE MEM_ALLOC_UNCACHE

void * MCP_MALLOC(unsigned int len, MEM_ALLOC_TYPE cache_type);
void MCP_FREE(void *data, unsigned int size, MEM_ALLOC_TYPE cache_type);

/************************************************************************
 *  Public functions
 ************************************************************************/
// DES
int DES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int DES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int DES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int DES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
// TDES
int TDES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int TDES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int TDES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int TDES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);

// AES_128
int AES_CBC_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4]);
int AES_ECB_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry);
int AES_CBC_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4]);
int AES_ECB_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry);
int AES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// AES_192
int AES_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// AES_256
int AES_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// CAM_128
int CAM_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int CAM_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int CAM_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
// CAM_192
int CAM_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// CAM_256
int CAM_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);

// Digest
int AES_hash(unsigned char * src_addr, int length, unsigned char * dst_addr, unsigned int block_size);
int SHA1_hash(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int iv[5]);
int SHA256_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[8]);
int SHA256_padding(char *p_buff, const unsigned int length);
int SHA1_padding(char* p_buff, unsigned long buff_len, const unsigned int data_len);
int MCP_HMAC_SHA256(unsigned char* keyIn, unsigned int keyIn_len, unsigned char* msg, unsigned int msg_len, unsigned char* p_out);
void reverse_signature( unsigned char * pSignature );

#endif
